@@ -113,7 +113,7 @@ pub struct TorClient<R: Runtime> {
     #[cfg(feature = "onion-service-client")]
     hsclient: HsClientConnector<R>,
     /// Circuit pool for providing onion services with circuits.
-    #[cfg(feature = "onion-service-service")]
+    #[cfg(any(feature = "onion-service-client", feature = "onion-service-service"))]
     hs_circ_pool: Arc<tor_circmgr::hspool::HsCircPool<R>>,
     /// The key manager.
     ///
@@ -683,7 +683,7 @@ impl<R: Runtime> TorClient<R> {
             pt_mgr,
             #[cfg(feature = "onion-service-client")]
             hsclient,
-            #[cfg(feature = "onion-service-service")]
+            #[cfg(any(feature = "onion-service-client", feature = "onion-service-service"))]
             hs_circ_pool,
             keymgr,
             guardmgr,
@@ -892,6 +892,14 @@ impl<R: Runtime> TorClient<R> {
             .reconfigure(new_config, how)
             .map_err(wrap_err)?;
 
+        #[cfg(all(
+            feature = "vanguards",
+            any(feature = "onion-service-client", feature = "onion-service-service")
+        ))]
+        self.hs_circ_pool
+            .reconfigure(new_config, how)
+            .map_err(wrap_err)?;
+
         self.dirmgr.reconfigure(&dir_cfg, how).map_err(wrap_err)?;
 
         let netparams = self.dirmgr.params();
@@ -1516,6 +1524,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
