@@ -25,6 +25,12 @@ pub use tor_guardmgr::bridge::BridgeConfigBuilder;
 #[cfg_attr(docsrs, doc(cfg(feature = "bridge-client")))]
 pub use tor_guardmgr::bridge::BridgeParseError;
 
+#[cfg(all(
+    feature = "vanguards",
+    any(feature = "onion-service-client", feature = "onion-service-service")
+))]
+pub use tor_guardmgr::vanguards::VanguardConfig;
+
 use tor_guardmgr::bridge::BridgeConfig;
 use tor_keymgr::config::arti::{ArtiNativeKeystoreConfig, ArtiNativeKeystoreConfigBuilder};
 
@@ -48,7 +54,7 @@ pub mod dir {
 /// Types for configuring pluggable transports.
 #[cfg(feature = "pt-client")]
 pub mod pt {
-    pub use tor_ptmgr::config::{ManagedTransportConfig, ManagedTransportConfigBuilder};
+    pub use tor_ptmgr::config::{TransportConfig, TransportConfigBuilder};
 }
 
 /// Types for configuring onion services.
@@ -57,6 +63,15 @@ pub mod onion_service {
     pub use tor_hsservice::config::{OnionServiceConfig, OnionServiceConfigBuilder};
 }
 
+/// Types for configuring vanguards.
+#[cfg(all(
+    feature = "vanguards",
+    any(feature = "onion-service-client", feature = "onion-service-service")
+))]
+pub mod vanguards {
+    pub use tor_guardmgr::vanguards::{VanguardConfig, VanguardConfigBuilder};
+}
+
 /// Configuration for client behavior relating to addresses.
 ///
 /// This type is immutable once constructed. To create an object of this type,
@@ -271,7 +286,7 @@ impl StorageConfig {
 /// A [`BridgesConfig`] configuration has the following pieces:
 ///    * A [`BridgeList`] of [`BridgeConfig`]s, which describes one or more bridges.
 ///    * An `enabled` boolean to say whether or not to use the listed bridges.
-///    * A list of [`pt::ManagedTransportConfig`]s.
+///    * A list of [`pt::TransportConfig`]s.
 ///
 /// # Example
 ///
@@ -286,7 +301,7 @@ impl StorageConfig {
 /// # fn demo() -> anyhow::Result<()> {
 /// use arti_client::config::{TorClientConfig, BridgeConfigBuilder, CfgPath};
 /// // Requires that the pt-client feature is enabled.
-/// use arti_client::config::pt::ManagedTransportConfigBuilder;
+/// use arti_client::config::pt::TransportConfigBuilder;
 ///
 /// let mut builder = TorClientConfig::builder();
 ///
@@ -313,7 +328,7 @@ impl StorageConfig {
 /// builder.bridges().bridges().push(bridge2_builder);
 ///
 /// // Now configure an obfs4 transport. (Requires the "pt-client" feature)
-/// let mut transport = ManagedTransportConfigBuilder::default();
+/// let mut transport = TransportConfigBuilder::default();
 /// transport
 ///     .protocols(vec!["obfs4".parse()?])
 ///     // Specify either the name or the absolute path of pluggable transport client binary, this
@@ -359,12 +374,12 @@ pub struct BridgesConfig {
 
 /// A list of configured transport binaries (type alias for macrology).
 #[cfg(feature = "pt-client")]
-type TransportConfigList = Vec<pt::ManagedTransportConfig>;
+type TransportConfigList = Vec<pt::TransportConfig>;
 
 #[cfg(feature = "pt-client")]
 define_list_builder_helper! {
     pub struct TransportConfigListBuilder {
-        transports: [pt::ManagedTransportConfigBuilder],
+        transports: [pt::TransportConfigBuilder],
     }
     built: TransportConfigList = transports;
     default = vec![];
@@ -373,7 +388,7 @@ define_list_builder_helper! {
 #[cfg(feature = "pt-client")]
 define_list_builder_accessors! {
     struct BridgesConfigBuilder {
-        pub transports: [pt::ManagedTransportConfigBuilder],
+        pub transports: [pt::TransportConfigBuilder],
     }
 }
 
@@ -621,6 +636,15 @@ pub struct TorClientConfig {
     #[builder(sub_builder)]
     #[builder_field_attr(serde(default))]
     pub(crate) stream_timeouts: StreamTimeoutConfig,
+
+    /// Information about vanguards.
+    #[cfg(all(
+        feature = "vanguards",
+        any(feature = "onion-service-client", feature = "onion-service-service")
+    ))]
+    #[builder(sub_builder)]
+    #[builder_field_attr(serde(default))]
+    pub(crate) vanguards: vanguards::VanguardConfig,
 }
 impl_standard_builder! { TorClientConfig }
 
@@ -639,10 +663,29 @@ fn convert_override_net_params(
     override_net_params
 }
 
-impl tor_circmgr::CircMgrConfig for TorClientConfig {}
+impl tor_circmgr::CircMgrConfig for TorClientConfig {
+    #[cfg(all(
+        feature = "vanguards",
+        any(feature = "onion-service-client", feature = "onion-service-service")
+    ))]
+    fn vanguard_config(&self) -> &tor_guardmgr::vanguards::VanguardConfig {
+        &self.vanguards
+    }
+}
 #[cfg(feature = "onion-service-client")]
 impl tor_hsclient::HsClientConnectorConfig for TorClientConfig {}
 
+#[cfg(any(feature = "onion-service-client", feature = "onion-service-service"))]
+impl tor_circmgr::hspool::HsCircPoolConfig for TorClientConfig {
+    #[cfg(all(
+        feature = "vanguards",
+        any(feature = "onion-service-client", feature = "onion-service-service")
+    ))]
+    fn vanguard_config(&self) -> &tor_guardmgr::vanguards::VanguardConfig {
+        &self.vanguards
+    }
+}
+
 impl AsRef<tor_guardmgr::fallback::FallbackList> for TorClientConfig {
     fn as_ref(&self) -> &tor_guardmgr::fallback::FallbackList {
         self.tor_network.fallback_caches()
@@ -759,6 +802,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -880,8 +924,8 @@ mod test {
         // here, so we'll just make sure it does _something_ plausible.
 
         let dflt = default_config_files().unwrap();
-        assert!(dflt[0].as_path().ends_with("arti.toml"));
-        assert!(dflt[1].as_path().ends_with("arti.d"));
+        assert!(dflt[0].as_path().unwrap().ends_with("arti.toml"));
+        assert!(dflt[1].as_path().unwrap().ends_with("arti.d"));
         assert_eq!(dflt.len(), 2);
     }
 
@@ -894,12 +938,26 @@ mod test {
             cfg
         };
 
-        let chk = |cfg: &TorClientConfigBuilder, expected: bool| {
-            assert_eq!(cfg.build().is_ok(), expected);
+        let chk = |cfg: &TorClientConfigBuilder, expected: Result<(), &str>| match (
+            cfg.build(),
+            expected,
+        ) {
+            (Ok(_), Ok(())) => {}
+            (Err(e), Err(ex)) => {
+                if !e.to_string().contains(ex) {
+                    panic!("\"{e}\" did not contain {ex}");
+                }
+            }
+            (Ok(_), Err(ex)) => {
+                panic!("Expected {ex} but cfg succeeded");
+            }
+            (Err(e), Ok(())) => {
+                panic!("Expected success but got error {e}")
+            }
         };
 
         let test_cases = [
-            ("# No bridges", true),
+            ("# No bridges", Ok(())),
             (
                 r#"
                     # No bridges but we still enabled bridges
@@ -907,7 +965,7 @@ mod test {
                     enabled = true
                     bridges = []
                 "#,
-                false,
+                Err("bridges.enabled=true, but no bridges defined"),
             ),
             (
                 r#"
@@ -918,7 +976,7 @@ mod test {
                         "192.0.2.83:80 $0bac39417268b96b9f514ef763fa6fba1a788956",
                     ]
                 "#,
-                true,
+                Ok(()),
             ),
             (
                 r#"
@@ -932,7 +990,31 @@ mod test {
                     protocols = ["obfs4"]
                     path = "obfs4proxy"
                 "#,
-                true,
+                Ok(()),
+            ),
+            (
+                r#"
+                    # One obfs4 bridge with unmanaged transport.
+                    [bridges]
+                    enabled = true
+                    bridges = [
+                        "obfs4 bridge.example.net:80 $0bac39417268b69b9f514e7f63fa6fba1a788958 ed25519:dGhpcyBpcyBbpmNyZWRpYmx5IHNpbGx5ISEhISEhISA iat-mode=1",
+                    ]
+                    [[bridges.transports]]
+                    protocols = ["obfs4"]
+                    proxy_addr = "127.0.0.1:31337"
+                "#,
+                Ok(()),
+            ),
+            (
+                r#"
+                    # Transport is both managed and unmanaged.
+                    [[bridges.transports]]
+                    protocols = ["obfs4"]
+                    path = "obfsproxy"
+                    proxy_addr = "127.0.0.1:9999"
+                "#,
+                Err("Cannot provide both path and proxy_addr"),
             ),
             (
                 r#"
@@ -947,7 +1029,7 @@ mod test {
                     protocols = ["obfs4"]
                     path = "obfs4proxy"
                 "#,
-                true,
+                Ok(()),
             ),
             (
                 r#"
@@ -959,7 +1041,7 @@ mod test {
                         "obfs4 bridge.example.net:80 $0bac39417268b69b9f514e7f63fa6fba1a788958 ed25519:dGhpcyBpcyBbpmNyZWRpYmx5IHNpbGx5ISEhISEhISA iat-mode=1",
                     ]
                 "#,
-                true,
+                Ok(()),
             ),
             (
                 r#"
@@ -970,7 +1052,7 @@ mod test {
                         "obfs4 bridge.example.net:80 $0bac39417268b69b9f514e7f63fa6fba1a788958 ed25519:dGhpcyBpcyBbpmNyZWRpYmx5IHNpbGx5ISEhISEhISA iat-mode=1",
                     ]
                 "#,
-                false,
+                Err("all bridges unusable due to lack of corresponding pluggable transport"),
             ),
             (
                 r#"
@@ -981,7 +1063,7 @@ mod test {
                         "obfs4 bridge.example.net:80 $0bac39417268b69b9f514e7f63fa6fba1a788958 ed25519:dGhpcyBpcyBbpmNyZWRpYmx5IHNpbGx5ISEhISEhISA iat-mode=1",
                     ]
                 "#,
-                true,
+                Ok(()),
             ),
             (
                 r#"
@@ -995,7 +1077,7 @@ mod test {
                         protocols = ["obfs4"]
                         path = "obfs4proxy"
                 "#,
-                true,
+                Ok(()),
             ),
         ];
 
