@@ -35,7 +35,7 @@ pub use hint::HintableError;
 pub struct Error {
     /// The actual error.
     ///
-    /// This field is exposed  via the `detail()` method only if the the
+    /// This field is exposed via the `detail()` method only if the
     /// `error_detail` feature is enabled. Using it will void your semver
     /// guarantee.
     #[source]
@@ -127,9 +127,18 @@ enum ErrorDetail {
 
     /// Error setting up the guard manager
     // TODO: should "guardmgr setup error" be its own type in tor-guardmgr?
-    #[error("Error setting up the circuit manager")]
+    #[error("Error setting up the guard manager")]
     GuardMgrSetup(#[source] tor_guardmgr::GuardMgrError),
 
+    /// Error setting up the guard manager
+    // TODO: should "vanguardmgr setup error" be its own type in tor-guardmgr?
+    #[cfg(all(
+        feature = "vanguards",
+        any(feature = "onion-service-client", feature = "onion-service-service")
+    ))]
+    #[error("Error setting up the vanguard manager")]
+    VanguardMgrSetup(#[source] tor_guardmgr::VanguardMgrError),
+
     /// Error setting up the circuit manager
     // TODO: should "circmgr setup error" be its own type in tor-circmgr?
     #[error("Error setting up the circuit manager")]
@@ -379,6 +388,11 @@ impl tor_error::HasKind for ErrorDetail {
             E::ExitTimeout => EK::RemoteNetworkTimeout,
             E::BootstrapRequired { .. } => EK::BootstrapRequired,
             E::GuardMgrSetup(e) => e.kind(),
+            #[cfg(all(
+                feature = "vanguards",
+                any(feature = "onion-service-client", feature = "onion-service-service")
+            ))]
+            E::VanguardMgrSetup(e) => e.kind(),
             #[cfg(feature = "bridge-client")]
             E::BridgeDescMgrSetup(e) => e.kind(),
             E::CircMgrSetup(e) => e.kind(),
@@ -536,6 +550,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
