@@ -23,7 +23,7 @@ pub(crate) use tor_config::{impl_standard_builder, ConfigBuildError, Listen};
 
 use crate::{LoggingConfig, LoggingConfigBuilder};
 
-/// Example file demonstrating our our configuration and the default options.
+/// Example file demonstrating our configuration and the default options.
 ///
 /// The options in this example file are all commented out;
 /// the actual defaults are done via builder attributes in all the Rust config structs.
@@ -301,6 +301,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -570,6 +571,17 @@ mod test {
             ],
         );
 
+        declare_exceptions(
+            None,
+            None, // TODO HS-VANGUARDS: add vanguards config to example
+            FeatureDependent,
+            &[
+                // Vanguards-specific settings
+                "vanguards",
+                "vanguards.mode",
+            ],
+        );
+
         out.sort();
 
         let dupes = out.iter().map(|exc| &exc.key).duplicates().collect_vec();
@@ -589,7 +601,9 @@ mod test {
     fn default_config() {
         use InExample::*;
 
-        let empty_config = config::Config::builder().build().unwrap();
+        let empty_config = tor_config::ConfigurationSources::new_empty()
+            .load()
+            .unwrap();
         let empty_config: ArtiCombinedConfig = tor_config::resolve(empty_config).unwrap();
 
         let default = (ArtiConfig::default(), TorClientConfig::default());
@@ -650,10 +664,14 @@ mod test {
         }
 
         let parses_to_defaults = |example: &str, which: WhichExample, uncommented: bool| {
-            let cfg = config::Config::builder()
-                .add_source(config::File::from_str(example, config::FileFormat::Toml))
-                .build()
-                .unwrap();
+            let cfg = {
+                let mut sources = tor_config::ConfigurationSources::new_empty();
+                sources.push_source(
+                    tor_config::ConfigurationSource::from_verbatim(example.to_string()),
+                    tor_config::sources::MustRead::MustRead,
+                );
+                sources.load().unwrap()
+            };
 
             // This tests that the example settings do not *contradict* the defaults.
             let results: ResolutionResults<ArtiCombinedConfig> =
@@ -1230,11 +1248,15 @@ example config file {which:?}, uncommented={uncommented:?}
 
         /// Make a TOML document of this section and parse it as a complete configuration.
         /// Panic if the section cannot be parsed.
-        fn parse(&self) -> config::Config {
+        fn parse(&self) -> tor_config::ConfigurationTree {
             let s = self.build_string();
             eprintln!("parsing\n  --\n{}\n  --", &s);
-            let c: toml::Value = toml::from_str(&s).expect(&s);
-            config::Config::try_from(&c).expect(&s)
+            let mut sources = tor_config::ConfigurationSources::new_empty();
+            sources.push_source(
+                tor_config::ConfigurationSource::from_verbatim(s.to_string()),
+                tor_config::sources::MustRead::MustRead,
+            );
+            sources.load().expect(&s)
         }
 
         fn resolve<R: tor_config::load::Resolvable>(&self) -> Result<R, ConfigResolveError> {
