@@ -30,6 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
+#![deny(clippy::unchecked_duration_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -218,7 +219,12 @@ async fn run<R: Runtime>(
     // NOTE: reconfigurable_modules stores the only strong references to these modules,
     // so we must keep the variable alive until the end of the function
     let weak_modules = reconfigurable_modules.iter().map(Arc::downgrade).collect();
-    reload_cfg::watch_for_config_changes(config_sources, &arti_config, &client, weak_modules)?;
+    reload_cfg::watch_for_config_changes(
+        client.runtime(),
+        config_sources,
+        &arti_config,
+        weak_modules,
+    )?;
 
     #[cfg(all(feature = "rpc", feature = "tokio"))]
     let rpc_mgr = {
@@ -406,7 +412,12 @@ where
                             .value_name("PORT")
                             .help("Port to listen on for DNS request (overrides the port in the config if specified).")
                     )
-            );
+            )
+            .subcommand_required(true)
+            .arg_required_else_help(true);
+
+    // When adding a subcommand, it may be necessary to add an entry in
+    // `maint/check-cli-help`, to the function `help_arg`.
 
     cfg_if::cfg_if! {
         // TODO (#1250): test the CLI.
@@ -426,15 +437,14 @@ where
                             .help("The nickname of the service")
                     )
                     .subcommand_required(true)
+                    .arg_required_else_help(true)
                     .subcommand(
                         Command::new("onion-name")
                             .about(
                                 "Print the .onion address of a hidden service",
                             )
                     )
-            )
-            .subcommand_required(true)
-            .arg_required_else_help(true);
+            );
         }
     }
 
