@@ -30,6 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
+#![deny(clippy::unchecked_duration_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -68,7 +69,7 @@ pub mod hspool;
 mod impls;
 pub mod isolation;
 mod mgr;
-pub mod path;
+pub(crate) mod path;
 mod preemptive;
 pub mod timeouts;
 mod usage;
@@ -96,6 +97,11 @@ pub use tor_guardmgr::{ExternalActivity, FirstHopId};
 use tor_persist::{FsStateMgr, StateMgr};
 use tor_rtcompat::scheduler::{TaskHandle, TaskSchedule};
 
+#[cfg(feature = "hs-common")]
+use crate::hspool::HsCircStubKind;
+#[cfg(all(feature = "vanguards", feature = "hs-common"))]
+use tor_guardmgr::vanguards::VanguardMgr;
+
 /// A Result type as returned from this crate.
 pub type Result<T> = std::result::Result<T, Error>;
 
@@ -179,7 +185,7 @@ impl<R: Runtime> CircMgr<R> {
         guardmgr: tor_guardmgr::GuardMgr<R>,
     ) -> Result<Arc<Self>>
     where
-        SM: tor_persist::StateMgr + Send + Sync + 'static,
+        SM: tor_persist::StateMgr + Clone + Send + Sync + 'static,
     {
         let preemptive = Arc::new(Mutex::new(PreemptiveCircuitPredictor::new(
             config.preemptive_circuits().clone(),
@@ -187,6 +193,9 @@ impl<R: Runtime> CircMgr<R> {
 
         guardmgr.set_filter(config.path_rules().build_guard_filter());
 
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let vanguardmgr = VanguardMgr::new(config.vanguard_config(), storage.clone())?;
+
         let storage_handle = storage.create_handle(PARETO_TIMEOUT_DATA_KEY);
 
         let builder = build::CircuitBuilder::new(
@@ -195,6 +204,8 @@ impl<R: Runtime> CircMgr<R> {
             config.path_rules().clone(),
             storage_handle,
             guardmgr,
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            vanguardmgr,
         );
         let mgr =
             mgr::AbstractCircMgr::new(builder, runtime.clone(), config.circuit_timing().clone());
@@ -296,6 +307,13 @@ impl<R: Runtime> CircMgr<R> {
         let retire_because_of_guardmgr =
             self.mgr.peek_builder().guardmgr().reconfigure(new_config)?;
 
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let retire_because_of_vanguardmgr = self
+            .mgr
+            .peek_builder()
+            .vanguardmgr()
+            .reconfigure(new_config.vanguard_config())?;
+
         let new_reachable = &new_config.path_rules().reachable_addrs;
         if new_reachable != &old_path_rules.reachable_addrs {
             let filter = new_config.path_rules().build_guard_filter();
@@ -307,6 +325,10 @@ impl<R: Runtime> CircMgr<R> {
             .at_least_as_permissive_as(&old_path_rules)
             || retire_because_of_guardmgr != tor_guardmgr::RetireCircuits::None;
 
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let discard_all_circuits = discard_all_circuits
+            || retire_because_of_vanguardmgr != tor_guardmgr::RetireCircuits::None;
+
         self.mgr
             .peek_builder()
             .set_path_config(new_config.path_rules().clone());
@@ -453,7 +475,7 @@ impl<R: Runtime> CircMgr<R> {
     /// future unless explicitly re-registered with a circuit manager.
     ///
     /// If `planned_target` is provided, then the circuit will be built so that
-    /// it does not share any family members with the the provided target.  (The
+    /// it does not share any family members with the provided target.  (The
     /// circuit _will not be_ extended to that target itself!)
     ///
     /// Used to implement onion service clients and services.
@@ -462,12 +484,14 @@ impl<R: Runtime> CircMgr<R> {
         &self,
         planned_target: Option<T>,
         dir: &NetDir,
+        kind: HsCircStubKind,
     ) -> Result<Arc<ClientCirc>>
     where
         T: IntoOwnedChanTarget,
     {
         let usage = TargetCircUsage::HsCircBase {
             compatible_with_target: planned_target.map(IntoOwnedChanTarget::to_owned),
+            kind,
         };
         let (_, client_circ) = self.mgr.launch_unmanaged(&usage, dir.into()).await?;
         Ok(client_circ)
@@ -863,6 +887,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
