@@ -1,17 +1,20 @@
 //! Code for building paths to an exit relay.
 
-use super::{MaybeOwnedRelay, TorPath};
-use crate::{DirInfo, Error, PathConfig, Result, TargetPort};
-use rand::Rng;
 use std::time::SystemTime;
-use tor_basic_utils::iter::FilterCount;
-use tor_error::{bad_api_usage, internal};
-#[cfg(feature = "geoip")]
-use tor_geoip::{CountryCode, HasCountryCode};
+
+use rand::Rng;
+
+use super::{AnonymousPathBuilder, TorPath};
+use crate::path::pick_path;
+use crate::{DirInfo, Error, PathConfig, Result, TargetPort};
+
 use tor_guardmgr::{GuardMgr, GuardMonitor, GuardUsable};
-use tor_linkspec::{HasRelayIds, OwnedChanTarget, RelayIdSet};
-use tor_netdir::{NetDir, Relay, SubnetConfig, WeightRole};
+use tor_linkspec::OwnedChanTarget;
+use tor_netdir::{NetDir, Relay};
+use tor_relay_selection::{RelayExclusion, RelaySelectionConfig, RelaySelector, RelayUsage};
 use tor_rtcompat::Runtime;
+#[cfg(feature = "geoip")]
+use {tor_geoip::CountryCode, tor_relay_selection::RelayRestriction};
 
 /// Internal representation of PathBuilder.
 enum ExitPathBuilderInner<'a> {
@@ -39,12 +42,8 @@ enum ExitPathBuilderInner<'a> {
         strict: bool,
     },
 
-    /// Request a path to any relay, even those that cannot exit.
-    // TODO: #785 may make this non-conditional.
-    #[cfg(feature = "hs-common")]
-    AnyRelay,
-
     /// Request a path that uses a given relay as exit node.
+    #[allow(unused)]
     ChosenExit(Relay<'a>),
 }
 
@@ -53,7 +52,7 @@ enum ExitPathBuilderInner<'a> {
 ///
 /// NOTE: The name of this type is no longer completely apt: given some circuits,
 /// it is happy to build a circuit ending at a non-exit.
-pub struct ExitPathBuilder<'a> {
+pub(crate) struct ExitPathBuilder<'a> {
     /// The inner ExitPathBuilder state.
     inner: ExitPathBuilderInner<'a>,
     /// If present, a "target" that every chosen relay must be able to share a circuit with with.
@@ -67,7 +66,7 @@ impl<'a> ExitPathBuilder<'a> {
     /// containing all the ports in `ports`.
     ///
     /// If the list of ports is empty, tries to get any exit relay at all.
-    pub fn from_target_ports(wantports: impl IntoIterator<Item = TargetPort>) -> Self {
+    pub(crate) fn from_target_ports(wantports: impl IntoIterator<Item = TargetPort>) -> Self {
         let ports: Vec<TargetPort> = wantports.into_iter().collect();
         if ports.is_empty() {
             return Self::for_any_exit();
@@ -86,7 +85,7 @@ impl<'a> ExitPathBuilder<'a> {
     ///
     /// If the list of ports is empty, it is disregarded.
     // TODO GEOIP: this method is hacky, and should be refactored.
-    pub fn in_given_country(
+    pub(crate) fn in_given_country(
         country: CountryCode,
         wantports: impl IntoIterator<Item = TargetPort>,
     ) -> Self {
@@ -98,18 +97,8 @@ impl<'a> ExitPathBuilder<'a> {
         }
     }
 
-    /// Create a new builder that will try to build a path with the given exit
-    /// relay as the last hop.
-    pub fn from_chosen_exit(exit_relay: Relay<'a>) -> Self {
-        Self {
-            inner: ExitPathBuilderInner::ChosenExit(exit_relay),
-            compatible_with: None,
-            require_stability: true,
-        }
-    }
-
     /// Create a new builder that will try to get any exit relay at all.
-    pub fn for_any_exit() -> Self {
+    pub(crate) fn for_any_exit() -> Self {
         Self {
             inner: ExitPathBuilderInner::AnyExit { strict: true },
             compatible_with: None,
@@ -117,23 +106,17 @@ impl<'a> ExitPathBuilder<'a> {
         }
     }
 
-    /// Create a new builder that will try to build a three-hop non-exit path
-    /// that is compatible with being extended to an optional given relay.
-    ///
-    /// (The provided relay is _not_ included in the built path: we only ensure
-    /// that the path we build does not have any features that would stop us
-    /// extending it to that relay as a fourth hop.)
-    ///
-    /// TODO: This doesn't seem to belong in a type called ExitPathBuilder.
-    /// Perhaps we should rename ExitPathBuilder, split it into multiple types,
-    /// or move this method.
-    #[cfg(feature = "hs-common")]
-    pub(crate) fn for_any_compatible_with(compatible_with: Option<OwnedChanTarget>) -> Self {
-        Self {
-            inner: ExitPathBuilderInner::AnyRelay,
-            compatible_with,
-            require_stability: true,
-        }
+    /// Try to create and return a path corresponding to the requirements of
+    /// this builder.
+    pub(crate) fn pick_path<R: Rng, RT: Runtime>(
+        &self,
+        rng: &mut R,
+        netdir: DirInfo<'a>,
+        guards: Option<&GuardMgr<RT>>,
+        config: &PathConfig,
+        now: SystemTime,
+    ) -> Result<(TorPath<'a>, Option<GuardMonitor>, Option<GuardUsable>)> {
+        pick_path(self, rng, netdir, guards, config, now)
     }
 
     /// Create a new builder that will try to get an exit relay, but which
@@ -152,294 +135,80 @@ impl<'a> ExitPathBuilder<'a> {
         self.require_stability = require_stability;
         self
     }
+}
 
-    /// Find a suitable exit node from either the chosen exit or from the network directory.
-    fn pick_exit<R: Rng>(
-        &self,
+impl<'a> AnonymousPathBuilder<'a> for ExitPathBuilder<'a> {
+    fn chosen_exit(&self) -> Option<&Relay<'_>> {
+        if let ExitPathBuilderInner::ChosenExit(e) = &self.inner {
+            Some(e)
+        } else {
+            None
+        }
+    }
+
+    fn compatible_with(&self) -> Option<&OwnedChanTarget> {
+        self.compatible_with.as_ref()
+    }
+
+    fn pick_exit<'s, R: Rng>(
+        &'s self,
         rng: &mut R,
         netdir: &'a NetDir,
-        guard: Option<&MaybeOwnedRelay<'a>>,
-        config: SubnetConfig,
-    ) -> Result<Relay<'a>> {
-        let mut can_share = FilterCount::default();
-        let mut correct_ports = FilterCount::default();
-        #[allow(unused_mut)]
-        let mut correct_country = FilterCount::default();
-        match &self.inner {
+        guard_exclusion: RelayExclusion<'a>,
+        rs_cfg: &RelaySelectionConfig<'_>,
+    ) -> Result<(Relay<'a>, RelayUsage)> {
+        let selector = match &self.inner {
             ExitPathBuilderInner::AnyExit { strict } => {
-                // TODO #504
-                let exit = netdir.pick_relay(rng, WeightRole::Exit, |r| {
-                    r.is_flagged_fast()
-                        && (!self.require_stability || r.is_flagged_stable())
-                        && can_share.count(r.policies_allow_some_port())
-                        && correct_ports.count(relays_can_share_circuit_opt(r, guard, config))
-                });
-                match (exit, strict) {
-                    (Some(exit), _) => return Ok(exit),
-                    (None, true) => {
-                        return Err(Error::NoExit {
-                            can_share,
-                            correct_ports,
-                            correct_country,
-                        })
-                    }
-                    (None, false) => {}
+                let mut selector =
+                    RelaySelector::new(RelayUsage::any_exit(rs_cfg), guard_exclusion);
+                if !strict {
+                    selector.mark_usage_flexible();
                 }
-
-                // Non-strict case.  Arguably this doesn't belong in
-                // ExitPathBuilder.
-                //
-                // Note that we use WeightRole::Exit here even though we don't
-                // care that this is actually an exit. That's because the
-                // purpose of this path is to learn average circuit build time
-                // information, so we want our distribution of possible final
-                // nodes to resemble the one that we would use for real
-                // circuits.
-                netdir
-                    // TODO #504
-                    .pick_relay(rng, WeightRole::Exit, |r| {
-                        r.is_flagged_fast()
-                            && (!self.require_stability || r.is_flagged_stable())
-                            && can_share.count(relays_can_share_circuit_opt(r, guard, config))
-                    })
-                    .ok_or(Error::NoExit {
-                        can_share,
-                        correct_ports,
-                        correct_country,
-                    })
+                selector
             }
+
             #[cfg(feature = "geoip")]
-            ExitPathBuilderInner::ExitInCountry { country, ports } => Ok(netdir
-                // TODO #504
-                .pick_relay(rng, WeightRole::Exit, |r| {
-                    r.is_flagged_fast()
-                        && (!self.require_stability || r.is_flagged_stable())
-                        && can_share.count(relays_can_share_circuit_opt(r, guard, config))
-                        && correct_ports.count(ports.iter().all(|p| p.is_supported_by(r)))
-                        && correct_country
-                            .count(r.country_code().map(|x| x == *country).unwrap_or(false))
-                })
-                .ok_or(Error::NoExit {
-                    can_share,
-                    correct_ports,
-                    correct_country,
-                })?),
+            ExitPathBuilderInner::ExitInCountry { country, ports } => {
+                let mut selector = RelaySelector::new(
+                    RelayUsage::exit_to_all_ports(rs_cfg, ports.clone()),
+                    guard_exclusion,
+                );
+                selector.push_restriction(RelayRestriction::require_country_code(*country));
+                selector
+            }
 
-            #[cfg(feature = "hs-common")]
-            ExitPathBuilderInner::AnyRelay => netdir
-                // TODO #504
-                .pick_relay(rng, WeightRole::Middle, |r| {
-                    r.is_flagged_fast()
-                        && (!self.require_stability || r.is_flagged_stable())
-                        && can_share.count(relays_can_share_circuit_opt(r, guard, config))
-                })
-                .ok_or(Error::NoExit {
-                    can_share,
-                    correct_ports,
-                    correct_country,
-                }),
-
-            ExitPathBuilderInner::WantsPorts(wantports) => Ok(netdir
-                // TODO #504
-                .pick_relay(rng, WeightRole::Exit, |r| {
-                    r.is_flagged_fast()
-                        && (!self.require_stability || r.is_flagged_stable())
-                        && can_share.count(relays_can_share_circuit_opt(r, guard, config))
-                        && correct_ports.count(wantports.iter().all(|p| p.is_supported_by(r)))
-                })
-                .ok_or(Error::NoExit {
-                    can_share,
-                    correct_ports,
-                    correct_country,
-                })?),
+            ExitPathBuilderInner::WantsPorts(wantports) => RelaySelector::new(
+                RelayUsage::exit_to_all_ports(rs_cfg, wantports.clone()),
+                guard_exclusion,
+            ),
 
             ExitPathBuilderInner::ChosenExit(exit_relay) => {
                 // NOTE that this doesn't check
                 // relays_can_share_circuit_opt(exit_relay,guard).  we
                 // already did that, sort of, in pick_path.
-                Ok(exit_relay.clone())
+                return Ok((exit_relay.clone(), RelayUsage::middle_relay(None)));
             }
+        };
+
+        let (relay, info) = selector.select_relay(rng, netdir);
+        let relay = relay.ok_or_else(|| Error::NoRelay {
+            path_kind: self.path_kind(),
+            role: "final hop",
+            problem: info.to_string(),
+        })?;
+        Ok((relay, RelayUsage::middle_relay(Some(selector.usage()))))
+    }
+
+    fn path_kind(&self) -> &'static str {
+        use ExitPathBuilderInner::*;
+        match &self.inner {
+            WantsPorts(_) => "exit circuit",
+            #[cfg(feature = "geoip")]
+            ExitInCountry { .. } => "country-specific exit circuit",
+            AnyExit { .. } => "testing circuit",
+            ChosenExit(_) => "circuit to a specific exit",
         }
     }
-
-    /// Try to create and return a path corresponding to the requirements of
-    /// this builder.
-    pub fn pick_path<R: Rng, RT: Runtime>(
-        &self,
-        rng: &mut R,
-        netdir: DirInfo<'a>,
-        guards: Option<&GuardMgr<RT>>,
-        config: &PathConfig,
-        _now: SystemTime,
-    ) -> Result<(TorPath<'a>, Option<GuardMonitor>, Option<GuardUsable>)> {
-        let netdir = match netdir {
-            DirInfo::Directory(d) => d,
-            _ => {
-                return Err(bad_api_usage!(
-                    "Tried to build a multihop path without a network directory"
-                )
-                .into())
-            }
-        };
-        let subnet_config = config.subnet_config();
-
-        let chosen_exit = if let ExitPathBuilderInner::ChosenExit(e) = &self.inner {
-            Some(e)
-        } else {
-            None
-        };
-        let path_is_fully_random = chosen_exit.is_none();
-
-        // TODO-SPEC: Because of limitations in guard selection, we have to
-        // pick the guard before the exit, which is not what our spec says.
-        let (guard, mon, usable) = match guards {
-            Some(guardmgr) => {
-                let mut b = tor_guardmgr::GuardUsageBuilder::default();
-                b.kind(tor_guardmgr::GuardUsageKind::Data);
-                if let Some(exit_relay) = chosen_exit {
-                    // TODO(nickm): Our way of building a family here is
-                    // somewhat questionable. We're only adding the ed25519
-                    // identities of the exit relay and its family to the
-                    // RelayId set.  That's fine for now, since we will only use
-                    // relays at this point if they have a known Ed25519
-                    // identity.  But if in the future the ed25519 identity
-                    // becomes optional, this will need to change.
-                    let mut family = RelayIdSet::new();
-                    family.insert(*exit_relay.id());
-                    // TODO(nickm): See "limitations" note on `known_family_members`.
-                    family.extend(netdir.known_family_members(exit_relay).map(|r| *r.id()));
-                    b.restrictions()
-                        .push(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
-                }
-                if let Some(avoid_target) = &self.compatible_with {
-                    let mut family = RelayIdSet::new();
-                    family.extend(avoid_target.identities().map(|id| id.to_owned()));
-                    if let Some(avoid_relay) = netdir.by_ids(avoid_target) {
-                        family.extend(netdir.known_family_members(&avoid_relay).map(|r| *r.id()));
-                    }
-                    b.restrictions()
-                        .push(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
-                }
-                let guard_usage = b.build().expect("Failed while building guard usage!");
-                let (guard, mut mon, usable) = guardmgr.select_guard(guard_usage)?;
-                let guard = if let Some(ct) = guard.as_circ_target() {
-                    // This is a bridge; we will not look for it in the network directory.
-                    MaybeOwnedRelay::from(ct.clone())
-                } else {
-                    // Look this up in the network directory: we expect to find a relay.
-                    guard
-                        .get_relay(netdir)
-                        .ok_or_else(|| {
-                            internal!(
-                                "Somehow the guardmgr gave us an unlisted guard {:?}!",
-                                guard
-                            )
-                        })?
-                        .into()
-                };
-                if !path_is_fully_random {
-                    // We were given a specific exit relay to use, and
-                    // the choice of exit relay might be forced by
-                    // something outside of our control.
-                    //
-                    // Therefore, we must not blame the guard for any failure
-                    // to complete the circuit.
-                    mon.ignore_indeterminate_status();
-                }
-                (guard, Some(mon), Some(usable))
-            }
-            None => {
-                let mut can_share = FilterCount::default();
-                let mut correct_usage = FilterCount::default();
-                let chosen_exit = chosen_exit.map(|relay| MaybeOwnedRelay::from(relay.clone()));
-                let entry = netdir
-                    // TODO #504
-                    .pick_relay(rng, WeightRole::Guard, |r| {
-                        can_share.count(relays_can_share_circuit_opt(
-                            r,
-                            chosen_exit.as_ref(),
-                            subnet_config,
-                        )) && correct_usage.count(r.is_suitable_as_guard())
-                    })
-                    .ok_or(Error::NoPath {
-                        role: "entry relay",
-                        can_share,
-                        correct_usage,
-                    })?;
-                (MaybeOwnedRelay::from(entry), None, None)
-            }
-        };
-
-        let exit: MaybeOwnedRelay = self
-            .pick_exit(rng, netdir, Some(&guard), subnet_config)?
-            .into();
-
-        let mut can_share = FilterCount::default();
-        let mut correct_usage = FilterCount::default();
-        let middle = netdir
-            // TODO #504
-            .pick_relay(rng, WeightRole::Middle, |r| {
-                r.is_flagged_fast()
-                    // TODO: if we intend to use this as an exit circuit, and
-                    // exit point only supports long-lived ports, we should
-                    // unconditionally require Stable here, since otherwise this
-                    // circuit isn't useful for exit purposes.
-                    && (!self.require_stability || r.is_flagged_stable())
-                    && can_share.count(
-                        relays_can_share_circuit(r, &exit, subnet_config)
-                            && relays_can_share_circuit(r, &guard, subnet_config),
-                    )
-                    && correct_usage.count(true)
-            })
-            .ok_or(Error::NoPath {
-                role: "middle relay",
-                can_share,
-                correct_usage,
-            })?;
-
-        Ok((
-            TorPath::new_multihop_from_maybe_owned(vec![
-                guard,
-                MaybeOwnedRelay::from(middle),
-                exit,
-            ]),
-            mon,
-            usable,
-        ))
-    }
-}
-
-/// Returns true if both relays can appear together in the same circuit.
-// TODO #789
-fn relays_can_share_circuit(
-    a: &Relay<'_>,
-    b: &MaybeOwnedRelay<'_>,
-    subnet_config: SubnetConfig,
-) -> bool {
-    // TODO #504
-    if let MaybeOwnedRelay::Relay(r) = b {
-        if a.in_same_family(r) {
-            return false;
-        };
-        // TODO: When bridge families are finally implemented (likely via
-        // proposal `321-happy-families.md`), we should move family
-        // functionality into CircTarget.
-    }
-
-    !subnet_config.any_addrs_in_same_subnet(a, b)
-}
-
-/// Helper: wraps relays_can_share_circuit but takes an option.
-// TODO #789
-fn relays_can_share_circuit_opt(
-    r1: &Relay<'_>,
-    r2: Option<&MaybeOwnedRelay<'_>>,
-    c: SubnetConfig,
-) -> bool {
-    match r2 {
-        Some(r2) => relays_can_share_circuit(r1, r2, c),
-        None => true,
-    }
 }
 
 #[cfg(test)]
@@ -448,6 +217,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -457,16 +227,45 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
-    use crate::path::{assert_same_path_when_owned, MaybeOwnedRelay, OwnedPath, TorPathInner};
+    use crate::path::{
+        assert_same_path_when_owned, MaybeOwnedRelay, OwnedPath, TorPath, TorPathInner,
+    };
     use crate::test::OptDummyGuardMgr;
     use std::collections::HashSet;
     use tor_basic_utils::test_rng::testing_rng;
     use tor_guardmgr::TestConfig;
     use tor_linkspec::{HasRelayIds, RelayIds};
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
-    use tor_netdir::testnet;
+    use tor_netdir::{testnet, SubnetConfig};
     use tor_rtcompat::SleepProvider;
 
+    impl<'a> ExitPathBuilder<'a> {
+        /// Create a new builder that will try to build a path with the given exit
+        /// relay as the last hop.
+        fn from_chosen_exit(exit_relay: Relay<'a>) -> Self {
+            Self {
+                inner: ExitPathBuilderInner::ChosenExit(exit_relay),
+                compatible_with: None,
+                require_stability: true,
+            }
+        }
+    }
+
+    /// Returns true if both relays can appear together in the same circuit.
+    fn relays_can_share_circuit(
+        a: &Relay<'_>,
+        b: &MaybeOwnedRelay<'_>,
+        subnet_config: SubnetConfig,
+    ) -> bool {
+        if let MaybeOwnedRelay::Relay(r) = b {
+            if a.in_same_family(r) {
+                return false;
+            };
+        }
+
+        !subnet_config.any_addrs_in_same_subnet(a, b)
+    }
+
     impl<'a> MaybeOwnedRelay<'a> {
         fn can_share_circuit(
             &self,
@@ -610,13 +409,13 @@ mod test {
         let outcome = ExitPathBuilder::from_target_ports(vec![TargetPort::ipv4(80)])
             .pick_path(&mut rng, dirinfo, guards, &config, now);
         assert!(outcome.is_err());
-        assert!(matches!(outcome, Err(Error::NoExit { .. })));
+        assert!(matches!(outcome, Err(Error::NoRelay { .. })));
 
         // For any exit
         let outcome =
             ExitPathBuilder::for_any_exit().pick_path(&mut rng, dirinfo, guards, &config, now);
         assert!(outcome.is_err());
-        assert!(matches!(outcome, Err(Error::NoExit { .. })));
+        assert!(matches!(outcome, Err(Error::NoRelay { .. })));
 
         // For any exit (non-strict, so this will work).
         let outcome = ExitPathBuilder::for_timeout_testing()
