@@ -1,7 +1,6 @@
 //! Code related to tracking what activities a circuit can be used for.
 
 use rand::Rng;
-use serde::{Deserialize, Serialize};
 use std::fmt::{self, Display};
 use std::sync::Arc;
 use std::time::SystemTime;
@@ -17,6 +16,8 @@ use tor_guardmgr::{GuardMgr, GuardMonitor, GuardUsable};
 use tor_netdir::Relay;
 use tor_netdoc::types::policy::PortPolicy;
 use tor_rtcompat::Runtime;
+#[cfg(feature = "hs-common")]
+use {crate::path::hspath::HsPathBuilder, crate::HsCircStubKind};
 
 #[cfg(feature = "specific-relay")]
 use tor_linkspec::{HasChanMethod, HasRelayIds};
@@ -37,10 +38,15 @@ pub(crate) type CountryCode = Void;
 #[cfg(any(feature = "specific-relay", feature = "hs-common"))]
 use tor_linkspec::OwnedChanTarget;
 
+#[cfg(all(feature = "vanguards", feature = "hs-common"))]
+use tor_guardmgr::vanguards::VanguardMgr;
+
 use crate::isolation::{IsolationHelper, StreamIsolation};
 use crate::mgr::{abstract_spec_find_supported, AbstractCirc, OpenEntry, RestrictionFailed};
 use crate::Result;
 
+pub use tor_relay_selection::TargetPort;
+
 /// An exit policy, as supported by the last hop of a circuit.
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) struct ExitPolicy {
@@ -50,50 +56,6 @@ pub(crate) struct ExitPolicy {
     v6: Arc<PortPolicy>,
 }
 
-/// A port that we want to connect to as a client.
-///
-/// Ordinarily, this is a TCP port, plus a flag to indicate whether we
-/// must support IPv4 or IPv6.
-#[derive(
-    Clone, Copy, Debug, Deserialize, Eq, PartialEq, Hash, PartialOrd, Ord, Serialize, Default,
-)]
-pub struct TargetPort {
-    /// True if this is a request to connect to an IPv6 address
-    ipv6: bool,
-    /// The port that the client wants to connect to
-    pub(crate) port: u16,
-}
-
-impl TargetPort {
-    /// Create a request to make sure that a circuit supports a given
-    /// ipv4 exit port.
-    pub fn ipv4(port: u16) -> TargetPort {
-        TargetPort { ipv6: false, port }
-    }
-
-    /// Create a request to make sure that a circuit supports a given
-    /// ipv6 exit port.
-    pub fn ipv6(port: u16) -> TargetPort {
-        TargetPort { ipv6: true, port }
-    }
-
-    /// Return true if this port is supported by the provided Relay.
-    pub fn is_supported_by(&self, r: &tor_netdir::Relay<'_>) -> bool {
-        // TODO #504
-        if self.ipv6 {
-            r.supports_exit_port_ipv6(self.port)
-        } else {
-            r.supports_exit_port_ipv4(self.port)
-        }
-    }
-}
-
-impl Display for TargetPort {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}{}", self.port, if self.ipv6 { "v6" } else { "v4" })
-    }
-}
-
 /// Set of requested target ports, mostly for use in error reporting
 ///
 /// Displays nicely.
@@ -128,7 +90,9 @@ impl Display for TargetPorts {
 impl ExitPolicy {
     /// Make a new exit policy from a given Relay.
     pub(crate) fn from_relay(relay: &Relay<'_>) -> Self {
-        // TODO #504
+        // TODO #504: it might be a good idea to lower this whole type into
+        // tor-netdir or tor-relay-selection.  That way we wouldn't need to
+        // invoke these Relay-specific methods in tor-circmgr.
         Self {
             v4: relay.ipv4_policy(),
             v6: relay.ipv6_policy(),
@@ -167,6 +131,8 @@ pub(crate) enum TargetCircUsage {
         /// Restrict the circuit to only exits in the provided country code.
         country_code: Option<CountryCode>,
         /// If true, all relays on this circuit need to have the Stable flag.
+        //
+        // TODO #504: It would be good to remove this field, if we can.
         require_stability: bool,
     },
     /// For a circuit is only used for the purpose of building it.
@@ -186,6 +152,7 @@ pub(crate) enum TargetCircUsage {
         /// The number of exit circuits needed for a port
         circs: usize,
         /// If true, all relays on this circuit need to have the Stable flag.
+        // TODO #504: It would be good to remove this field, if we can.
         require_stability: bool,
     },
     /// Use for BEGINDIR-based non-anonymous directory connections to a particular target,
@@ -204,6 +171,8 @@ pub(crate) enum TargetCircUsage {
         /// circuit with this target (without, for example, violating any
         /// family restrictions).
         compatible_with_target: Option<OwnedChanTarget>,
+        /// The kind of stub circuit to build.
+        kind: HsCircStubKind,
     },
 }
 
@@ -225,6 +194,8 @@ pub(crate) enum SupportedCircUsage {
         /// Country code the exit is in, or `None` if no country could be determined.
         country_code: Option<CountryCode>,
         /// Whether every relay in this circuit has the "Stable" flag.
+        //
+        // TODO #504: It would be good to remove this field, if we can.
         all_relays_stable: bool,
     },
     /// This circuit is not suitable for any usage.
@@ -248,6 +219,7 @@ impl TargetCircUsage {
         rng: &mut R,
         netdir: crate::DirInfo<'a>,
         guards: Option<&GuardMgr<RT>>,
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))] vanguards: &VanguardMgr,
         config: &crate::PathConfig,
         now: SystemTime,
     ) -> Result<(
@@ -368,13 +340,18 @@ impl TargetCircUsage {
             #[cfg(feature = "hs-common")]
             TargetCircUsage::HsCircBase {
                 compatible_with_target,
+                kind,
             } => {
-                let (path, mon, usable) =
-                    ExitPathBuilder::for_any_compatible_with(compatible_with_target.clone())
-                        // TODO: We don't actually require stability if this is a
-                        // HsDir circuit: but at this point, we can't tell.
-                        .require_stability(true)
-                        .pick_path(rng, netdir, guards, config, now)?;
+                let path_builder = HsPathBuilder::new(compatible_with_target.clone(), *kind);
+                cfg_if::cfg_if! {
+                    if #[cfg(all(feature = "vanguards", feature = "hs-common"))] {
+                        let (path, mon, usable) = path_builder
+                            .pick_path_with_vanguards::<_, RT>(rng, netdir, guards, vanguards, config, now)?;
+                    } else {
+                        let (path, mon, usable) = path_builder
+                            .pick_path::<_, RT>(rng, netdir, guards, config, now)?;
+                    }
+                };
                 let usage = SupportedCircUsage::HsOnly;
                 Ok((path, usage, mon, usable))
             }
@@ -413,7 +390,8 @@ impl crate::mgr::AbstractSpec for SupportedCircUsage {
                     require_stability,
                 },
             ) => {
-                // TODO #504
+                // TODO #504: These calculations don't touch Relays, but they
+                // seem like they should be done using the types of tor-relay-selection.
                 i1.as_ref()
                     .map(|i1| i1.compatible_same_type(i2))
                     .unwrap_or(true)
@@ -434,7 +412,8 @@ impl crate::mgr::AbstractSpec for SupportedCircUsage {
                     ..
                 },
             ) => {
-                // TODO #504
+                // TODO #504: It would be good to simply remove stability
+                // calculation from tor-circmgr.
                 if *require_stability && !all_relays_stable {
                     return false;
                 }
@@ -443,7 +422,8 @@ impl crate::mgr::AbstractSpec for SupportedCircUsage {
                     // for new streams that don't share it.
                     return false;
                 }
-                // TODO #504
+                // TODO #504: Similarly, it would be good to have exit port
+                // calculation done elsewhere.
                 if let Some(p) = port {
                     policy.allows_port(*p)
                 } else {
@@ -554,6 +534,8 @@ pub(crate) mod test {
     use tor_basic_utils::test_rng::testing_rng;
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_netdir::testnet;
+    #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+    use tor_persist::TestingStateMgr;
 
     impl IsolationTokenEq for TargetCircUsage {
         fn isol_eq(&self, other: &Self) -> bool {
@@ -895,9 +877,20 @@ pub(crate) mod test {
         // building code a lot more closely in the tests for TorPath
         // and friends.
 
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let vanguards = VanguardMgr::new(&Default::default(), TestingStateMgr::default()).unwrap();
+
         // First, a one-hop directory circuit
         let (p_dir, u_dir, _, _) = TargetCircUsage::Dir
-            .build_path(&mut rng, di, guards, &config, now)
+            .build_path(
+                &mut rng,
+                di,
+                guards,
+                #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+                &vanguards,
+                &config,
+                now,
+            )
             .unwrap();
         assert!(matches!(u_dir, SupportedCircUsage::Dir));
         assert_eq!(p_dir.len(), 1);
@@ -916,7 +909,15 @@ pub(crate) mod test {
             require_stability: false,
         };
         let (p_exit, u_exit, _, _) = exit_usage
-            .build_path(&mut rng, di, guards, &config, now)
+            .build_path(
+                &mut rng,
+                di,
+                guards,
+                #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+                &vanguards,
+                &config,
+                now,
+            )
             .unwrap();
         assert!(matches!(
             u_exit,
@@ -930,7 +931,15 @@ pub(crate) mod test {
 
         // Now try testing circuits.
         let (path, usage, _, _) = TargetCircUsage::TimeoutTesting
-            .build_path(&mut rng, di, guards, &config, now)
+            .build_path(
+                &mut rng,
+                di,
+                guards,
+                #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+                &vanguards,
+                &config,
+                now,
+            )
             .unwrap();
         let path = match OwnedPath::try_from(&path).unwrap() {
             OwnedPath::ChannelOnly(_) => panic!("Impossible path type."),
@@ -972,8 +981,19 @@ pub(crate) mod test {
         let guards: OptDummyGuardMgr<'_> = None;
         let now = SystemTime::now();
 
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let vanguards = VanguardMgr::new(&Default::default(), TestingStateMgr::default()).unwrap();
+
         let (path, usage, _, _) = TargetCircUsage::TimeoutTesting
-            .build_path(&mut rng, di, guards, &config, now)
+            .build_path(
+                &mut rng,
+                di,
+                guards,
+                #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+                &vanguards,
+                &config,
+                now,
+            )
             .unwrap();
         assert_eq!(path.len(), 3);
         assert_isoleq!(usage, SupportedCircUsage::NoUsage);
