@@ -4,6 +4,7 @@ use tor_linkspec::ChanTarget;
 // TODO(nickm): Conceivably, this type should be exposed from a lower-level crate than
 // tor-netdoc.
 use tor_netdoc::types::policy::AddrPortPattern;
+use tor_relay_selection::{LowLevelRelayPredicate, RelayRestriction, RelaySelector, RelayUsage};
 
 /// An object specifying which relays are eligible to be guards.
 ///
@@ -80,9 +81,20 @@ impl GuardFilter {
         use tor_netdir::{RelayWeight, WeightRole};
         let mut guard_bw: RelayWeight = 0.into();
         let mut permitted_bw: RelayWeight = 0.into();
+        // TODO #504: This is an unaccompanied RelayUsage, and is therefore a
+        // bit suspect.  We should consider whether we like this behavior,
+        // or whether we should convert it into a RelaySelector.
+        //
+        // It is not _too_ bad, however, since we're only looking at the
+        // fraction of the relays that might be guards; we won't use these
+        // relays without later choosing them via a RelaySelector.  Nonetheless,
+        // it would be better to construct a RelaySelector.
+        let usage = RelayUsage::new_guard();
+
+        // TODO: There is a case to be made for converting "permitted by this
+        // address-port filter?" into a RelayRestriction.
         for relay in netdir.relays() {
-            // TODO #504
-            if relay.is_suitable_as_guard() {
+            if usage.low_level_predicate_permits_relay(&relay) {
                 let w = netdir.relay_weight(&relay, WeightRole::Guard);
                 guard_bw += w;
                 if self.permits(&relay) {
@@ -93,12 +105,30 @@ impl GuardFilter {
 
         permitted_bw.checked_div(guard_bw).unwrap_or(1.0)
     }
+
+    /// Update `selector` with all the restrictions from this filter.
+    pub(crate) fn add_to_selector(&self, selector: &mut RelaySelector) {
+        // TODO #504: There is a case to be made that we should refactor
+        // `tor-guardmgr` crate so that GuardFilter no longer exists
+        // independently, but instead is just a part of RelaySelector.
+        //
+        // But before we do that, we should let the rest of #504 settle.
+        for filt in &self.filters {
+            selector.push_restriction(match filt {
+                SingleFilter::ReachableAddrs(addrs) => {
+                    RelayRestriction::require_address(addrs.clone())
+                }
+            });
+        }
+    }
 }
 
 impl SingleFilter {
     /// Return true if this filter permits the provided target.
     fn permits<C: ChanTarget>(&self, target: &C) -> bool {
         match self {
+            // TODO: This is partially duplicated with tor-relay-selection,
+            // but (for now) that only covers Relays, not general ChanTargets.
             SingleFilter::ReachableAddrs(patterns) => {
                 patterns.iter().any(|pat| {
                     match target.chan_method().socket_addrs() {
@@ -159,6 +189,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -190,6 +221,6 @@ mod test {
             f.push_reachable_addresses(vec!["1.0.0.0/8:*".parse().unwrap()]);
             f
         };
-        assert_float_eq!(net_1_only.frac_bw_permitted(&nd), 54.0 / 330.0, abs <= TOL);
+        assert_float_eq!(net_1_only.frac_bw_permitted(&nd), 0.28, abs <= TOL);
     }
 }
