@@ -30,6 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
+#![deny(clippy::unchecked_duration_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -58,6 +59,7 @@ use tor_error::internal;
 use tor_linkspec::{OwnedChanTarget, OwnedCircTarget, RelayId, RelayIdSet};
 use tor_netdir::NetDirProvider;
 use tor_proto::ClockSkew;
+use tor_units::BoundedInt32;
 use tracing::{debug, info, trace, warn};
 
 use tor_config::impl_standard_builder;
@@ -82,6 +84,8 @@ mod pending;
 mod sample;
 mod skew;
 mod util;
+#[cfg(feature = "vanguards")]
+pub mod vanguards;
 
 #[cfg(not(feature = "bridge-client"))]
 #[path = "bridge_disabled.rs"]
@@ -101,6 +105,10 @@ pub use ids::FirstHopId;
 pub use pending::{GuardMonitor, GuardStatus, GuardUsable};
 pub use skew::SkewEstimate;
 
+#[cfg(feature = "vanguards")]
+#[cfg_attr(docsrs, doc(cfg(feature = "vanguards")))]
+pub use vanguards::VanguardMgrError;
+
 use pending::{PendingRequest, RequestId};
 use sample::{GuardSet, Universe, UniverseRef};
 
@@ -848,7 +856,7 @@ impl GuardMgrInner {
     }
 
     /// Update the status of all guards in the active set, based on the passage
-    /// of time, our configuration, and and the relevant Universe for our active
+    /// of time, our configuration, and the relevant Universe for our active
     /// set.
     fn update(&mut self, wallclock: SystemTime, now: Instant) {
         self.with_opt_netdir(|this, netdir| {
@@ -1835,12 +1843,63 @@ pub enum GuardRestriction {
     AvoidAllIds(RelayIdSet),
 }
 
+/// The kind of vanguards to use.
+#[derive(Debug, Default, Clone, Copy, Eq, PartialEq, Serialize, Deserialize)] //
+#[derive(derive_more::Display)] //
+#[serde(rename_all = "lowercase")]
+#[cfg(feature = "vanguards")]
+#[non_exhaustive]
+pub enum VanguardMode {
+    /// "Lite" vanguards.
+    #[default]
+    #[display(fmt = "lite")]
+    Lite,
+    /// "Full" vanguards.
+    #[display(fmt = "full")]
+    Full,
+    /// Vanguards are disabled.
+    #[display(fmt = "disabled")]
+    Disabled,
+}
+
+#[cfg(feature = "vanguards")]
+impl VanguardMode {
+    /// Build a `VanguardMode` from a [`NetParameters`] parameter.
+    ///
+    /// Used for converting [`vanguards_enabled`](NetParameters::vanguards_enabled)
+    /// or [`vanguards_hs_service`](NetParameters::vanguards_hs_service)
+    /// to the corresponding `VanguardMode`.
+    #[allow(dead_code)] // TODO HS-VANGUARDS
+    pub(crate) fn from_net_parameter(val: BoundedInt32<0, 2>) -> Self {
+        match val.get() {
+            0 => VanguardMode::Disabled,
+            1 => VanguardMode::Lite,
+            2 => VanguardMode::Full,
+            _ => unreachable!("BoundedInt32 was not bounded?!"),
+        }
+    }
+}
+
+/// The kind of vanguards to use.
+#[derive(Debug, Default, Clone, Copy, Eq, PartialEq, Serialize, Deserialize)] //
+#[derive(derive_more::Display)] //
+#[serde(rename_all = "lowercase")]
+#[cfg(not(feature = "vanguards"))]
+#[non_exhaustive]
+pub enum VanguardMode {
+    /// Vanguards are disabled.
+    #[default]
+    #[display(fmt = "disabled")]
+    Disabled,
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
