@@ -1,17 +1,8 @@
 //! Configuration information for onion services.
 
-use base64ct::{Base64Unpadded, Encoding as _};
-use derive_adhoc::Adhoc;
-use derive_builder::Builder;
-use serde::{Deserialize, Serialize};
-use std::path::PathBuf;
-use tor_cell::relaycell::hs::est_intro;
-use tor_config::ConfigBuildError;
-use tor_error::into_internal;
-use tor_hscrypto::pk::HsClientDescEncKey;
-use tor_llcrypto::pk::curve25519;
+use crate::internal_prelude::*;
 
-use crate::HsNickname;
+use tor_cell::relaycell::hs::est_intro;
 
 /// Configuration for one onion service.
 #[derive(Debug, Clone, Builder, Eq, PartialEq)]
@@ -49,12 +40,11 @@ pub struct OnionServiceConfig {
     /// <https://spec.torproject.org/rend-spec/introduction-protocol.html#EST_INTRO_DOS_EXT>.
     #[builder(default)]
     rate_limit_at_intro: Option<TokenBucketConfig>,
-    // TODO #1124 Implement max_concurrent_streams_per_circuit
-    // /// How many streams will we allow to be open at once for a single circuit on
-    // /// this service?
-    // #[builder(default = "65535")]
-    // max_concurrent_streams_per_circuit: u32,
 
+    /// How many streams will we allow to be open at once for a single circuit on
+    /// this service?
+    #[builder(default = "65535")]
+    max_concurrent_streams_per_circuit: u32,
     // TODO POW: The POW items are disabled for now, since they aren't implemented.
     // /// If true, we will require proof-of-work when we're under heavy load.
     // // enable_pow: bool,
@@ -181,6 +171,9 @@ impl OnionServiceConfig {
             // IPT manager's "new configuration" select arm handles this,
             // by replacing IPTs if necessary.
             rate_limit_at_intro: simply_update,
+
+            // We extract this on every introduction request.
+            max_concurrent_streams_per_circuit: simply_update,
         }
 
         Ok(other)
@@ -197,6 +190,13 @@ impl OnionServiceConfig {
                 "somehow built an un-validated rate-limit-at-intro"
             ))?)
     }
+
+    /// Return a RequestFilter based on this configuration.
+    pub(crate) fn filter_settings(&self) -> crate::rend_handshake::RequestFilter {
+        crate::rend_handshake::RequestFilter {
+            max_concurrent_streams: self.max_concurrent_streams_per_circuit as usize,
+        }
+    }
 }
 
 impl OnionServiceConfigBuilder {
@@ -301,7 +301,9 @@ pub struct DescEncryptionConfig {
 #[non_exhaustive]
 pub enum AuthorizedClientConfig {
     /// A directory full of authorized public keys.
-    DirectoryOfKeys(PathBuf),
+    ///
+    /// Currently, only valid unicode pathnames are supported.
+    DirectoryOfKeys(String),
     /// A single authorized public key.
     Curve25519Key(HsClientDescEncKey),
 }
@@ -309,7 +311,7 @@ pub enum AuthorizedClientConfig {
 impl std::fmt::Display for AuthorizedClientConfig {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::DirectoryOfKeys(pb) => write!(f, "dir:{}", pb.display()),
+            Self::DirectoryOfKeys(pb) => write!(f, "dir:{}", pb),
             Self::Curve25519Key(key) => write!(
                 f,
                 "curve25519:{}",
