@@ -1,4 +1,4 @@
-//! [`KeySpecifier`](tor_keymgr::KeySpecifier) implementations for hidden service keys.
+//! [`KeySpecifier`] implementations for hidden service keys.
 //!
 //! Some of these `KeySpecifier`s represent time-bound keys (that are only valid
 //! as long as their time period is relevant). Time-bound keys are expired (removed)
@@ -9,20 +9,7 @@
 //! For TP-based keys, that involves deriving [`HsTimePeriodKeySpecifier`]
 //! and adding a call to `remove_if_expired!` in [`expire_publisher_keys`].
 
-use std::fmt::Debug;
-
-use derive_adhoc::{define_derive_adhoc, Adhoc};
-use derive_more::Constructor;
-
-use tor_error::internal;
-use tor_hscrypto::time::TimePeriod;
-use tor_keymgr::derive_adhoc_template_KeySpecifier;
-use tor_keymgr::KeyMgr;
-use tor_keymgr::KeySpecifierComponentViaDisplayFromStr;
-use tor_netdir::HsDirParams;
-
-use crate::HsNickname;
-use crate::IptLocalId;
+use crate::internal_prelude::*;
 
 /// Keys that are used by publisher, which relate to our HS and a TP
 ///
@@ -201,6 +188,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
