@@ -20,6 +20,8 @@ use ssh_key::private::PrivateKey;
 use ssh_key::{LineEnding, PublicKey};
 use walkdir::WalkDir;
 
+use tor_basic_utils::PathExt as _;
+
 use super::SshKeyData;
 
 /// The Arti key store.
@@ -242,8 +244,8 @@ impl Keystore for ArtiNativeKeystore {
                         /* This error should be impossible. */
                         tor_error::internal!(
                             "found key {} outside of keystore_dir {}?!",
-                            path.display(),
-                            self.keystore_dir.as_path().display()
+                            path.display_lossy(),
+                            self.keystore_dir.as_path().display_lossy()
                         )
                     })?;
 
@@ -296,6 +298,7 @@ mod tests {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -406,7 +409,7 @@ mod tests {
             err.to_string(),
             format!(
                 "Invalid path or permissions on {} while attempting to Init",
-                keystore_dir.path().display()
+                keystore_dir.path().display_lossy()
             ),
             "expected keystore init failure (perms = {:o})",
             mode
@@ -468,7 +471,7 @@ mod tests {
                 key_store
                     .key_path(&key_spec, ed_key_type)
                     .unwrap()
-                    .display()
+                    .display_lossy()
             ),
         );
     }
