@@ -15,9 +15,14 @@
 //!    well-formed? For open streams, the streams themselves handle this check.
 //!    For half-closed streams, the reactor handles it by calling
 //!    `consume_checked_msg()`.
-use super::streammap::{ShouldSendEnd, StreamEnt};
+
+pub(super) mod syncview;
+
+use super::handshake::RelayCryptLayerProtocol;
+use super::streammap::{EndSentStreamEnt, OpenStreamEnt, ShouldSendEnd, StreamEntMut};
 use super::MutableState;
 use crate::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
+use crate::circuit::handshake::{BoxedClientLayer, HandshakeRole};
 use crate::circuit::unique_id::UniqId;
 use crate::circuit::{
     sendme, streammap, CircParameters, Create2Wrap, CreateFastWrap, CreateHandshakeWrap,
@@ -39,10 +44,13 @@ use std::marker::PhantomData;
 use std::pin::Pin;
 use tor_cell::chancell::msg::{AnyChanMsg, HandshakeType, Relay};
 use tor_cell::relaycell::msg::{AnyRelayMsg, End, Sendme};
-use tor_cell::relaycell::{AnyRelayMsgOuter, RelayCmd, StreamId, UnparsedRelayCell};
+use tor_cell::relaycell::{
+    AnyRelayMsgOuter, RelayCellDecoder, RelayCellFormat, RelayCellFormatTrait, RelayCellFormatV0,
+    RelayCmd, StreamId, UnparsedRelayMsg,
+};
 #[cfg(feature = "hs-service")]
 use {
-    crate::stream::{DataCmdChecker, IncomingStreamRequest},
+    crate::stream::{DataCmdChecker, IncomingStreamRequest, IncomingStreamRequestFilter},
     tor_cell::relaycell::msg::Begin,
 };
 
@@ -178,6 +186,8 @@ pub(super) enum CtrlMsg {
     /// INTRODUCE and RENDEZVOUS messages.)
     #[cfg(feature = "hs-common")]
     ExtendVirtual {
+        /// Which relay cell format to use for this hop.
+        relay_cell_format: RelayCellFormat,
         /// The cryptographic algorithms and keys to use when communicating with
         /// the newly added hop.
         #[educe(Debug(ignore))]
@@ -205,7 +215,7 @@ pub(super) enum CtrlMsg {
         /// SENDME cells once we've read enough out of the other end. If it *does* block, we
         /// can assume someone is trying to send us more cells than they should, and abort
         /// the stream.
-        sender: mpsc::Sender<UnparsedRelayCell>,
+        sender: mpsc::Sender<UnparsedRelayMsg>,
         /// A channel to receive messages to send on this stream from.
         rx: mpsc::Receiver<AnyRelayMsg>,
         /// Oneshot channel to notify on completion, with the allocated stream ID.
@@ -235,13 +245,17 @@ pub(super) enum CtrlMsg {
     #[cfg(feature = "hs-service")]
     AwaitStreamRequest {
         /// A channel for sending information about an incoming stream request.
-        incoming_sender: mpsc::Sender<IncomingStreamRequestContext>,
+        incoming_sender: mpsc::Sender<StreamReqInfo>,
         /// A `CmdChecker` to keep track of which message types are acceptable.
         cmd_checker: AnyCmdChecker,
         /// Oneshot channel to notify on completion.
         done: ReactorResultChannel<()>,
         /// The hop that is allowed to create streams.
         hop_num: HopNum,
+        /// A filter used to check requests before passing them on.
+        #[educe(Debug(ignore))]
+        #[cfg(feature = "hs-service")]
+        filter: Box<dyn IncomingStreamRequestFilter>,
     },
     /// Send a given control message on this circuit.
     #[cfg(feature = "send-control-msg")]
@@ -281,6 +295,7 @@ pub(super) enum CtrlMsg {
     /// (tests only) Add a hop to the list of hops on this circuit, with dummy cryptography.
     #[cfg(test)]
     AddFakeHop {
+        relay_cell_format: RelayCellFormat,
         fwd_lasthop: bool,
         rev_lasthop: bool,
         params: CircParameters,
@@ -324,6 +339,8 @@ pub(super) struct CircHop {
     /// NOTE: Control messages could potentially add unboundedly to this, although that's
     ///       not likely to happen (and isn't triggereable from the network, either).
     outbound: VecDeque<(bool, AnyRelayMsgOuter)>,
+    /// Decodes relay cells received from this hop.
+    inbound: RelayCellDecoder,
 }
 
 /// An indicator on what we should do when we receive a cell for a circuit.
@@ -337,12 +354,13 @@ enum CellStatus {
 
 impl CircHop {
     /// Create a new hop.
-    pub(super) fn new(initial_window: u16) -> Self {
+    pub(super) fn new(format: RelayCellFormat, initial_window: u16) -> Self {
         CircHop {
             map: streammap::StreamMap::new(),
             recvwindow: sendme::CircRecvWindow::new(1000),
             sendwindow: sendme::CircSendWindow::new(initial_window),
             outbound: VecDeque::new(),
+            inbound: RelayCellDecoder::new(format),
         }
     }
 }
@@ -388,7 +406,7 @@ pub(super) trait MetaCellHandler: Send {
     fn handle_msg(
         &mut self,
         cx: &mut Context<'_>,
-        msg: UnparsedRelayCell,
+        msg: UnparsedRelayMsg,
         reactor: &mut Reactor,
     ) -> Result<MetaCellDisposition>;
 }
@@ -431,6 +449,8 @@ where
     unique_id: UniqId,
     /// The hop we're expecting the EXTENDED2 cell to come back from.
     expected_hop: HopNum,
+    /// The relay cell format we intend to use for this hop.
+    relay_cell_format: RelayCellFormat,
     /// A oneshot channel that we should inform when we are done with this extend operation.
     operation_finished: Option<oneshot::Sender<Result<()>>>,
     /// `PhantomData` used to make the other type parameters required for a circuit extension
@@ -461,6 +481,7 @@ where
     #[allow(clippy::blocks_in_conditions)]
     fn begin(
         cx: &mut Context<'_>,
+        relay_cell_format: RelayCellFormat,
         peer_id: OwnedChanTarget,
         handshake_id: HandshakeType,
         key: &H::KeyType,
@@ -506,6 +527,7 @@ where
                 expected_hop: hop,
                 operation_finished: None,
                 phantom: Default::default(),
+                relay_cell_format,
             })
         })() {
             Ok(mut result) => {
@@ -525,7 +547,7 @@ where
     /// This is a separate function to simplify the error-handling work of handle_msg().
     fn extend_circuit(
         &mut self,
-        msg: UnparsedRelayCell,
+        msg: UnparsedRelayMsg,
         reactor: &mut Reactor,
     ) -> Result<MetaCellDisposition> {
         let msg = msg
@@ -559,6 +581,7 @@ where
         // If we get here, it succeeded.  Add a new hop to the circuit.
         let (layer_fwd, layer_back, binding) = layer.split();
         reactor.add_hop(
+            self.relay_cell_format,
             path::HopDetail::Relay(self.peer_id.clone()),
             Box::new(layer_fwd),
             Box::new(layer_back),
@@ -584,7 +607,7 @@ where
     fn handle_msg(
         &mut self,
         _cx: &mut Context<'_>,
-        msg: UnparsedRelayCell,
+        msg: UnparsedRelayMsg,
         reactor: &mut Reactor,
     ) -> Result<MetaCellDisposition> {
         let status = self.extend_circuit(msg, reactor);
@@ -722,7 +745,7 @@ pub struct Reactor {
 /// Information about an incoming stream request.
 #[cfg(feature = "hs-service")]
 #[derive(Debug)]
-pub(super) struct IncomingStreamRequestContext {
+pub(super) struct StreamReqInfo {
     /// The [`IncomingStreamRequest`].
     pub(super) req: IncomingStreamRequest,
     /// The ID of the stream being requested.
@@ -736,21 +759,26 @@ pub(super) struct IncomingStreamRequestContext {
     // incoming stream request from two separate hops.  (There is only one that's valid.)
     pub(super) hop_num: HopNum,
     /// A channel for receiving messages from this stream.
-    pub(super) receiver: mpsc::Receiver<UnparsedRelayCell>,
+    pub(super) receiver: mpsc::Receiver<UnparsedRelayMsg>,
     /// A channel for sending messages to be sent on this stream.
     pub(super) msg_tx: mpsc::Sender<AnyRelayMsg>,
 }
 
 /// Data required for handling an incoming stream request.
 #[cfg(feature = "hs-service")]
-#[derive(Debug)]
+#[derive(educe::Educe)]
+#[educe(Debug)]
 struct IncomingStreamRequestHandler {
     /// A sender for sharing information about an incoming stream request.
-    incoming_sender: mpsc::Sender<IncomingStreamRequestContext>,
+    incoming_sender: mpsc::Sender<StreamReqInfo>,
     /// A [`AnyCmdChecker`] for validating incoming stream requests.
     cmd_checker: AnyCmdChecker,
     /// The hop to expect incoming stream requests from.
     hop_num: HopNum,
+    /// An [`IncomingStreamRequestFilter`] for checking whether the user wants
+    /// this request, or wants to reject it immediately.
+    #[educe(Debug(ignore))]
+    filter: Box<dyn IncomingStreamRequestFilter>,
 }
 
 impl Reactor {
@@ -917,10 +945,10 @@ impl Reactor {
                         }
                         let hop = &mut self.hops[i];
                         // Look at all of the streams on this hop.
-                        for (id, stream) in hop.map.inner().iter_mut() {
-                            if let StreamEnt::Open {
+                        for (id, stream) in hop.map.iter_mut() {
+                            if let StreamEntMut::Open(OpenStreamEnt {
                                 rx, send_window, ..
-                            } = stream
+                            }) = stream
                             {
                                 // Do the stream and hop send windows allow us to obtain and
                                 // send something?
@@ -931,14 +959,14 @@ impl Reactor {
                                         Poll::Ready(Some(m)) => {
                                             stream_relaycells.push((
                                                 hop_num,
-                                                AnyRelayMsgOuter::new(Some(*id), m),
+                                                AnyRelayMsgOuter::new(Some(id), m),
                                             ));
                                         }
                                         Poll::Ready(None) => {
                                             // Stream receiver was dropped; close the stream.
                                             // We can't close it here though due to borrowck; that
                                             // will happen later.
-                                            streams_to_close.push((hop_num, *id));
+                                            streams_to_close.push((hop_num, id));
                                         }
                                         Poll::Pending => {}
                                     }
@@ -1005,12 +1033,13 @@ impl Reactor {
             CtrlMsg::Shutdown => self.handle_shutdown(),
             #[cfg(test)]
             CtrlMsg::AddFakeHop {
+                relay_cell_format: format,
                 fwd_lasthop,
                 rev_lasthop,
                 params,
                 done,
             } => {
-                self.handle_add_fake_hop(fwd_lasthop, rev_lasthop, &params, done);
+                self.handle_add_fake_hop(format, fwd_lasthop, rev_lasthop, &params, done);
                 Ok(())
             }
             _ => {
@@ -1071,6 +1100,7 @@ impl Reactor {
     #[cfg(test)]
     fn handle_add_fake_hop(
         &mut self,
+        format: RelayCellFormat,
         fwd_lasthop: bool,
         rev_lasthop: bool,
         params: &CircParameters,
@@ -1088,6 +1118,7 @@ impl Reactor {
         let rev = Box::new(DummyCrypto::new(rev_lasthop));
         let binding = None;
         self.add_hop(
+            format,
             path::HopDetail::Relay(dummy_peer_id),
             fwd,
             rev,
@@ -1100,11 +1131,11 @@ impl Reactor {
     /// Helper: create the first hop of a circuit.
     ///
     /// This is parameterized not just on the RNG, but a wrapper object to
-    /// build the right kind of create cell, a handshake object to perform
-    /// the cryptographic cryptographic handshake, and a layer type to
-    /// handle relay crypto after this hop is built.
-    async fn create_impl<L, FWD, REV, H, W, M>(
+    /// build the right kind of create cell, and a handshake object to perform
+    /// the cryptographic handshake.
+    async fn create_impl<H, W, M>(
         &mut self,
+        cell_protocol: RelayCryptLayerProtocol,
         recvcreated: oneshot::Receiver<CreateResponse>,
         wrap: &W,
         key: &H::KeyType,
@@ -1112,9 +1143,6 @@ impl Reactor {
         msg: &M,
     ) -> Result<()>
     where
-        L: CryptInit + ClientLayer<FWD, REV> + 'static + Send,
-        FWD: OutboundClientLayer + 'static + Send,
-        REV: InboundClientLayer + 'static + Send,
         H: ClientHandshake + HandshakeAuxDataHandler,
         W: CreateHandshakeWrap,
         H::KeyGen: KeyGenerator,
@@ -1147,18 +1175,20 @@ impl Reactor {
 
         H::handle_server_aux_data(self, params, &server_msg)?;
 
-        let layer = L::construct(keygen)?;
+        let relay_cell_format = cell_protocol.relay_cell_format();
+        let BoxedClientLayer { fwd, back, binding } =
+            cell_protocol.construct_layers(HandshakeRole::Initiator, keygen)?;
 
         trace!("{}: Handshake complete; circuit created.", self.unique_id);
 
-        let (layer_fwd, layer_back, binding) = layer.split();
         let peer_id = self.channel.target().clone();
 
         self.add_hop(
+            relay_cell_format,
             path::HopDetail::Relay(peer_id),
-            Box::new(layer_fwd),
-            Box::new(layer_back),
-            Some(binding),
+            fwd,
+            back,
+            binding,
             params,
         );
         Ok(())
@@ -1175,15 +1205,11 @@ impl Reactor {
         recvcreated: oneshot::Receiver<CreateResponse>,
         params: &CircParameters,
     ) -> Result<()> {
+        // In a CREATE_FAST handshake, we can't negotiate a format other than this.
+        let protocol = RelayCryptLayerProtocol::Tor1(RelayCellFormat::V0);
         let wrap = CreateFastWrap;
-        self.create_impl::<Tor1RelayCrypto, _, _, CreateFastClient, _, _>(
-            recvcreated,
-            &wrap,
-            &(),
-            params,
-            &(),
-        )
-        .await
+        self.create_impl::<CreateFastClient, _, _>(protocol, recvcreated, &wrap, &(), params, &())
+            .await
     }
 
     /// Use the ntor handshake to connect to the first hop of this circuit.
@@ -1197,6 +1223,9 @@ impl Reactor {
         pubkey: NtorPublicKey,
         params: &CircParameters,
     ) -> Result<()> {
+        // In an ntor handshake, we can't negotiate a format other than this.
+        let relay_cell_protocol = RelayCryptLayerProtocol::Tor1(RelayCellFormat::V0);
+
         // Exit now if we have an Ed25519 or RSA identity mismatch.
         let target = RelayIds::builder()
             .ed_identity(ed_identity)
@@ -1208,7 +1237,8 @@ impl Reactor {
         let wrap = Create2Wrap {
             handshake_type: HandshakeType::NTOR,
         };
-        self.create_impl::<Tor1RelayCrypto, _, _, NtorClient, _, _>(
+        self.create_impl::<NtorClient, _, _>(
+            relay_cell_protocol,
             recvcreated,
             &wrap,
             &pubkey,
@@ -1236,6 +1266,9 @@ impl Reactor {
             .expect("Unable to build RelayIds");
         self.channel.check_match(&target)?;
 
+        // TODO: Add support for negotiating other formats.
+        let relay_cell_protocol = RelayCryptLayerProtocol::Tor1(RelayCellFormat::V0);
+
         // TODO: Set client extensions. e.g. request congestion control
         // if specified in `params`.
         let client_extensions = [];
@@ -1243,7 +1276,9 @@ impl Reactor {
         let wrap = Create2Wrap {
             handshake_type: HandshakeType::NTOR_V3,
         };
-        self.create_impl::<Tor1RelayCrypto, _, _, NtorV3Client, _, _>(
+
+        self.create_impl::<NtorV3Client, _, _>(
+            relay_cell_protocol,
             recvcreated,
             &wrap,
             &pubkey,
@@ -1256,13 +1291,14 @@ impl Reactor {
     /// Add a hop to the end of this circuit.
     fn add_hop(
         &mut self,
+        format: RelayCellFormat,
         peer_id: path::HopDetail,
         fwd: Box<dyn OutboundClientLayer + 'static + Send>,
         rev: Box<dyn InboundClientLayer + 'static + Send>,
         binding: Option<CircuitBinding>,
         params: &CircParameters,
     ) {
-        let hop = crate::circuit::reactor::CircHop::new(params.initial_send_window());
+        let hop = crate::circuit::reactor::CircHop::new(format, params.initial_send_window());
         self.hops.push(hop);
         self.crypto_in.add_layer(rev);
         self.crypto_out.add_layer(fwd);
@@ -1276,7 +1312,7 @@ impl Reactor {
         &mut self,
         cx: &mut Context<'_>,
         hopnum: HopNum,
-        msg: UnparsedRelayCell,
+        msg: UnparsedRelayMsg,
     ) -> Result<CellStatus> {
         // SENDME cells and TRUNCATED get handled internally by the circuit.
 
@@ -1500,18 +1536,22 @@ impl Reactor {
             if let Some(stream_id) = stream_id {
                 // We need to decrement the stream-level sendme window.
                 // Stream data cells should only be dequeued and fed into this function if
-                // the window is above zero, so we don't need to worry about enqueuing things.
-                if let Some(window) = hop.map.get_mut(stream_id).and_then(StreamEnt::send_window) {
-                    window.take(&())?;
-                } else {
-                    warn!(
-                        "{}: sending a relay cell for non-existent or non-open stream with ID {}!",
-                        self.unique_id, stream_id
-                    );
-                    return Err(Error::CircProto(format!(
-                        "tried to send a relay cell on non-open stream {}",
-                        sv(stream_id),
-                    )));
+                // the window is above zero, so we don't need to worry about
+                // enqueuing things.
+                match hop.map.get_mut(stream_id) {
+                    Some(StreamEntMut::Open(OpenStreamEnt { send_window, .. })) => {
+                        send_window.take(&())?;
+                    }
+                    _ => {
+                        warn!(
+                            "{}: sending a relay cell for non-existent or non-open stream with ID {}!",
+                            self.unique_id, stream_id
+                        );
+                        return Err(Error::CircProto(format!(
+                            "tried to send a relay cell on non-open stream {}",
+                            sv(stream_id),
+                        )));
+                    }
                 }
             }
         }
@@ -1562,8 +1602,13 @@ impl Reactor {
                 params,
                 done,
             } => {
-                let extender = CircuitExtender::<NtorClient, Tor1RelayCrypto, _, _>::begin(
+                // ntor handshake only supports V0.
+                /// Local type alias to ensure consistency below.
+                type Rcf = RelayCellFormatV0;
+
+                let extender = CircuitExtender::<NtorClient, Tor1RelayCrypto<Rcf>, _, _>::begin(
                     cx,
+                    Rcf::FORMAT,
                     peer_id,
                     HandshakeType::NTOR,
                     &public_key,
@@ -1583,11 +1628,16 @@ impl Reactor {
                 params,
                 done,
             } => {
+                // TODO #1067: support negotiating other formats.
+                /// Local type alias to ensure consistency below.
+                type Rcf = RelayCellFormatV0;
+
                 // TODO: Set extensions, e.g. based on `params`.
                 let client_extensions = [];
 
-                let extender = CircuitExtender::<NtorV3Client, Tor1RelayCrypto, _, _>::begin(
+                let extender = CircuitExtender::<NtorV3Client, Tor1RelayCrypto<Rcf>, _, _>::begin(
                     cx,
+                    Rcf::FORMAT,
                     peer_id,
                     HandshakeType::NTOR_V3,
                     &public_key,
@@ -1602,6 +1652,7 @@ impl Reactor {
             #[cfg(feature = "hs-common")]
             #[allow(unreachable_code)]
             CtrlMsg::ExtendVirtual {
+                relay_cell_format: format,
                 cell_crypto,
                 params,
                 done,
@@ -1612,7 +1663,7 @@ impl Reactor {
                 // describe why the virtual hop was added, or something?
                 let peer_id = path::HopDetail::Virtual;
 
-                self.add_hop(peer_id, outbound, inbound, binding, &params);
+                self.add_hop(format, peer_id, outbound, inbound, binding, &params);
                 let _ = done.send(Ok(()));
             }
             CtrlMsg::BeginStream {
@@ -1648,6 +1699,7 @@ impl Reactor {
                 incoming_sender,
                 hop_num,
                 done,
+                filter,
             } => {
                 // TODO: At some point we might want to add a CtrlMsg for
                 // de-registering the handler.  See comments on `allow_stream_requests`.
@@ -1655,6 +1707,7 @@ impl Reactor {
                     incoming_sender,
                     cmd_checker,
                     hop_num,
+                    filter,
                 };
 
                 let ret = self.set_incoming_stream_req_handler(handler);
@@ -1700,11 +1753,20 @@ impl Reactor {
             }
             #[cfg(test)]
             CtrlMsg::AddFakeHop {
+                relay_cell_format,
                 fwd_lasthop,
                 rev_lasthop,
                 params,
                 done,
-            } => self.handle_add_fake_hop(fwd_lasthop, rev_lasthop, &params, done),
+            } => {
+                self.handle_add_fake_hop(
+                    relay_cell_format,
+                    fwd_lasthop,
+                    rev_lasthop,
+                    &params,
+                    done,
+                );
+            }
             #[cfg(test)]
             CtrlMsg::QuerySendWindow { hop, done } => {
                 let _ = done.send(if let Some(hop) = self.hop_mut(hop) {
@@ -1731,7 +1793,7 @@ impl Reactor {
         cx: &mut Context<'_>,
         hopnum: HopNum,
         message: AnyRelayMsg,
-        sender: mpsc::Sender<UnparsedRelayCell>,
+        sender: mpsc::Sender<UnparsedRelayMsg>,
         rx: mpsc::Receiver<AnyRelayMsg>,
         cmd_checker: AnyCmdChecker,
     ) -> Result<StreamId> {
@@ -1825,10 +1887,21 @@ impl Reactor {
             tag_copy.copy_from_slice(tag);
             tag_copy
         };
-        // Put the cell into a format where we can make sense of it.
-        let msg = UnparsedRelayCell::from_body(body.into());
 
-        let c_t_w = sendme::cell_counts_towards_windows(&msg);
+        // Decode the cell.
+        let decode_res = self
+            .hop_mut(hopnum)
+            .ok_or_else(|| {
+                Error::from(internal!(
+                    "Trying to decode cell from nonexistent hop {:?}",
+                    hopnum
+                ))
+            })?
+            .inbound
+            .decode(body.into())
+            .map_err(|e| Error::from_bytes_err(e, "relay cell"))?;
+
+        let c_t_w = decode_res.cmds().any(sendme::cmd_counts_towards_windows);
 
         // Decrement the circuit sendme windows, and see if we need to
         // send a sendme cell.
@@ -1860,7 +1933,37 @@ impl Reactor {
                 .put();
         }
 
-        // If this cell wants/refuses to have a Stream ID, does it
+        let (mut msgs, incomplete) = decode_res.into_parts();
+        while let Some(msg) = msgs.next() {
+            let msg_status = self.handle_relay_msg(cx, hopnum, c_t_w, msg)?;
+            match msg_status {
+                CellStatus::Continue => (),
+                CellStatus::CleanShutdown => {
+                    for msg in msgs {
+                        debug!(
+                            "{id}: Ignoring relay msg received after triggering shutdown: {msg:?}",
+                            id = self.unique_id
+                        );
+                    }
+                    if let Some(incomplete) = incomplete {
+                        debug!("{id}: Ignoring parial relay msg received after triggering shutdown: {incomplete:?}", id=self.unique_id);
+                    }
+                    return Ok(CellStatus::CleanShutdown);
+                }
+            }
+        }
+        Ok(CellStatus::Continue)
+    }
+
+    /// Handle a single incoming relay message.
+    fn handle_relay_msg(
+        &mut self,
+        cx: &mut Context<'_>,
+        hopnum: HopNum,
+        cell_counts_toward_windows: bool,
+        msg: UnparsedRelayMsg,
+    ) -> Result<CellStatus> {
+        // If this msg wants/refuses to have a Stream ID, does it
         // have/not have one?
         let cmd = msg.cmd();
         let streamid = msg.stream_id();
@@ -1882,13 +1985,13 @@ impl Reactor {
             .hop_mut(hopnum)
             .ok_or_else(|| Error::CircProto("Cell from nonexistent hop!".into()))?;
         match hop.map.get_mut(streamid) {
-            Some(StreamEnt::Open {
+            Some(StreamEntMut::Open(OpenStreamEnt {
                 sink,
                 send_window,
                 dropped,
                 cmd_checker,
                 ..
-            }) => {
+            })) => {
                 // The stream for this message exists, and is open.
 
                 if msg.cmd() == RelayCmd::SENDME {
@@ -1914,7 +2017,7 @@ impl Reactor {
                             sv(streamid),
                         )));
                     }
-                    if e.is_disconnected() && c_t_w {
+                    if e.is_disconnected() && cell_counts_toward_windows {
                         // the other side of the stream has gone away; remember
                         // that we received a cell that we couldn't queue for it.
                         //
@@ -1927,7 +2030,7 @@ impl Reactor {
                 }
             }
             #[cfg(feature = "hs-service")]
-            Some(StreamEnt::EndSent { .. })
+            Some(StreamEntMut::EndSent(_))
                 if matches!(
                     msg.cmd(),
                     RelayCmd::BEGIN | RelayCmd::BEGIN_DIR | RelayCmd::RESOLVE
@@ -1937,9 +2040,9 @@ impl Reactor {
                 // message, just remove the old stream from the map and stop waiting for a
                 // response
                 hop.map.ending_msg_received(streamid)?;
-                self.handle_incoming_stream_request(cx, msg, streamid, hopnum)?;
+                return self.handle_incoming_stream_request(cx, msg, streamid, hopnum);
             }
-            Some(StreamEnt::EndSent { half_stream, .. }) => {
+            Some(StreamEntMut::EndSent(EndSentStreamEnt { half_stream, .. })) => {
                 // We sent an end but maybe the other side hasn't heard.
 
                 match half_stream.handle_msg(msg)? {
@@ -1970,14 +2073,17 @@ impl Reactor {
     fn handle_incoming_stream_request(
         &mut self,
         cx: &mut Context<'_>,
-        msg: UnparsedRelayCell,
+        msg: UnparsedRelayMsg,
         stream_id: StreamId,
         hop_num: HopNum,
-    ) -> Result<()> {
+    ) -> Result<CellStatus> {
+        use syncview::ClientCircSyncView;
         use tor_cell::relaycell::msg::EndReason;
         use tor_error::into_internal;
         use tor_log_ratelim::log_ratelim;
 
+        // We need to construct this early so that we don't double-borrow &mut self
+
         let Some(handler) = self.incoming_stream_req_handler.as_mut() else {
             return Err(Error::CircProto(
                 "Cannot handle BEGIN cells on this circuit".into(),
@@ -2010,7 +2116,7 @@ impl Reactor {
         if message_closes_stream {
             hop.map.ending_msg_received(stream_id)?;
 
-            return Ok(());
+            return Ok(CellStatus::Continue);
         }
 
         let begin = msg
@@ -2020,6 +2126,30 @@ impl Reactor {
 
         let req = IncomingStreamRequest::Begin(begin);
 
+        {
+            use crate::stream::IncomingStreamRequestDisposition::*;
+
+            let ctx = crate::stream::IncomingStreamRequestContext { request: &req };
+            let view = ClientCircSyncView::new(&self.hops);
+
+            match handler.filter.as_mut().disposition(&ctx, &view)? {
+                Accept => {}
+                CloseCircuit => return Ok(CellStatus::CleanShutdown),
+                RejectRequest(end) => {
+                    let end_msg = AnyRelayMsgOuter::new(Some(stream_id), end.into());
+                    self.send_relay_cell(cx, hop_num, false, end_msg)?;
+                    return Ok(CellStatus::Continue);
+                }
+            }
+        }
+
+        // TODO: Sadly, we need to look up `&mut hop` yet again,
+        // since we needed to pass `&self.hops` by reference to our filter above. :(
+        let hop = self
+            .hops
+            .get_mut(Into::<usize>::into(hop_num))
+            .ok_or(Error::CircuitClosed)?;
+
         let (sender, receiver) = mpsc::channel(STREAM_READER_BUFFER);
         let (msg_tx, msg_rx) = mpsc::channel(super::CIRCUIT_BUFFER_SIZE);
 
@@ -2030,7 +2160,7 @@ impl Reactor {
 
         let outcome = handler
             .incoming_sender
-            .try_send(IncomingStreamRequestContext {
+            .try_send(StreamReqInfo {
                 req,
                 stream_id,
                 hop_num,
@@ -2079,7 +2209,7 @@ impl Reactor {
             }
         }
 
-        Ok(())
+        Ok(CellStatus::Continue)
     }
 
     /// Helper: process a destroy cell.
