@@ -264,15 +264,15 @@ impl InboundClientCrypt {
 }
 
 /// Standard Tor relay crypto, as instantiated for RELAY cells.
-pub(crate) type Tor1RelayCrypto =
-    tor1::CryptStatePair<tor_llcrypto::cipher::aes::Aes128Ctr, tor_llcrypto::d::Sha1>;
+pub(crate) type Tor1RelayCrypto<RCF> =
+    tor1::CryptStatePair<tor_llcrypto::cipher::aes::Aes128Ctr, tor_llcrypto::d::Sha1, RCF>;
 
 /// Standard Tor relay crypto, as instantiated for the HSv3 protocol.
 ///
 /// (The use of SHA3 is ridiculously overkill.)
 #[cfg(feature = "hs-common")]
-pub(crate) type Tor1Hsv3RelayCrypto =
-    tor1::CryptStatePair<tor_llcrypto::cipher::aes::Aes256Ctr, tor_llcrypto::d::Sha3_256>;
+pub(crate) type Tor1Hsv3RelayCrypto<RCF> =
+    tor1::CryptStatePair<tor_llcrypto::cipher::aes::Aes256Ctr, tor_llcrypto::d::Sha3_256, RCF>;
 
 /// An implementation of Tor's current relay cell cryptography.
 ///
@@ -284,11 +284,14 @@ pub(crate) type Tor1Hsv3RelayCrypto =
 /// I am calling this design `tor1`; it does not have a generally recognized
 /// name.
 pub(crate) mod tor1 {
+    use std::marker::PhantomData;
+
     use crate::crypto::binding::CIRC_BINDING_LEN;
 
     use super::*;
     use cipher::{KeyIvInit, StreamCipher};
     use digest::Digest;
+    use tor_cell::relaycell::{RelayCellFields, RelayCellFormatTrait};
     use typenum::Unsigned;
 
     /// A CryptState represents one layer of shared cryptographic state between
@@ -304,7 +307,7 @@ pub(crate) mod tor1 {
     /// It is parameterized on a stream cipher and a digest type: most circuits
     /// will use AES-128-CTR and SHA1, but v3 onion services use AES-256-CTR and
     /// SHA-3.
-    pub(crate) struct CryptState<SC: StreamCipher, D: Digest + Clone> {
+    pub(crate) struct CryptState<SC: StreamCipher, D: Digest + Clone, RCF: RelayCellFormatTrait> {
         /// Stream cipher for en/decrypting cell bodies.
         ///
         /// This cipher is the one keyed with Kf or Kb in the spec.
@@ -315,21 +318,25 @@ pub(crate) mod tor1 {
         digest: D,
         /// Most recent digest value generated by this crypto.
         last_digest_val: GenericArray<u8, D::OutputSize>,
+        /// The format used for relay cells at this layer.
+        relay_cell_format: PhantomData<RCF>,
     }
 
     /// A pair of CryptStates shared between a client and a relay, one for the
     /// outbound (away from the client) direction, and one for the inbound
     /// (towards the client) direction.
-    pub(crate) struct CryptStatePair<SC: StreamCipher, D: Digest + Clone> {
+    pub(crate) struct CryptStatePair<SC: StreamCipher, D: Digest + Clone, RCF: RelayCellFormatTrait> {
         /// State for en/decrypting cells sent away from the client.
-        fwd: CryptState<SC, D>,
+        fwd: CryptState<SC, D, RCF>,
         /// State for en/decrypting cells sent towards the client.
-        back: CryptState<SC, D>,
+        back: CryptState<SC, D, RCF>,
         /// A circuit binding key.
         binding: CircuitBinding,
     }
 
-    impl<SC: StreamCipher + KeyIvInit, D: Digest + Clone> CryptInit for CryptStatePair<SC, D> {
+    impl<SC: StreamCipher + KeyIvInit, D: Digest + Clone, RCF: RelayCellFormatTrait> CryptInit
+        for CryptStatePair<SC, D, RCF>
+    {
         fn seed_len() -> usize {
             SC::KeySize::to_usize() * 2 + D::OutputSize::to_usize() * 2 + CIRC_BINDING_LEN
         }
@@ -363,11 +370,13 @@ pub(crate) mod tor1 {
                 cipher: SC::new(kf.into(), &Default::default()),
                 digest: D::new().chain_update(df),
                 last_digest_val: GenericArray::default(),
+                relay_cell_format: PhantomData,
             };
             let back = CryptState {
                 cipher: SC::new(kb.into(), &Default::default()),
                 digest: D::new().chain_update(db),
                 last_digest_val: GenericArray::default(),
+                relay_cell_format: PhantomData,
             };
             let binding = CircuitBinding::try_from(binding_key)?;
 
@@ -375,20 +384,30 @@ pub(crate) mod tor1 {
         }
     }
 
-    impl<SC, D> ClientLayer<CryptState<SC, D>, CryptState<SC, D>> for CryptStatePair<SC, D>
+    impl<SC, D, RCF> ClientLayer<CryptState<SC, D, RCF>, CryptState<SC, D, RCF>>
+        for CryptStatePair<SC, D, RCF>
     where
         SC: StreamCipher,
         D: Digest + Clone,
+        RCF: RelayCellFormatTrait,
     {
-        fn split(self) -> (CryptState<SC, D>, CryptState<SC, D>, CircuitBinding) {
+        fn split(
+            self,
+        ) -> (
+            CryptState<SC, D, RCF>,
+            CryptState<SC, D, RCF>,
+            CircuitBinding,
+        ) {
             (self.fwd, self.back, self.binding)
         }
     }
 
-    impl<SC: StreamCipher, D: Digest + Clone> RelayCrypt for CryptStatePair<SC, D> {
+    impl<SC: StreamCipher, D: Digest + Clone, RCF: RelayCellFormatTrait> RelayCrypt
+        for CryptStatePair<SC, D, RCF>
+    {
         fn originate(&mut self, cell: &mut RelayCellBody) {
             let mut d_ignored = GenericArray::default();
-            cell.set_digest(&mut self.back.digest, &mut d_ignored);
+            cell.set_digest::<_, RCF>(&mut self.back.digest, &mut d_ignored);
         }
         fn encrypt_inbound(&mut self, cell: &mut RelayCellBody) {
             // This is describe in tor-spec 5.5.3.1, "Relaying Backward at Onion Routers"
@@ -398,13 +417,15 @@ pub(crate) mod tor1 {
             // This is describe in tor-spec 5.5.2.2, "Relaying Forward at Onion Routers"
             self.fwd.cipher.apply_keystream(cell.as_mut());
             let mut d_ignored = GenericArray::default();
-            cell.recognized(&mut self.fwd.digest, &mut d_ignored)
+            cell.is_recognized::<_, RCF>(&mut self.fwd.digest, &mut d_ignored)
         }
     }
 
-    impl<SC: StreamCipher, D: Digest + Clone> OutboundClientLayer for CryptState<SC, D> {
+    impl<SC: StreamCipher, D: Digest + Clone, RCF: RelayCellFormatTrait> OutboundClientLayer
+        for CryptState<SC, D, RCF>
+    {
         fn originate_for(&mut self, cell: &mut RelayCellBody) -> &[u8] {
-            cell.set_digest(&mut self.digest, &mut self.last_digest_val);
+            cell.set_digest::<_, RCF>(&mut self.digest, &mut self.last_digest_val);
             self.encrypt_outbound(cell);
             // Note that we truncate the authentication tag here if we are using
             // a digest with a more-than-20-byte length.
@@ -417,12 +438,14 @@ pub(crate) mod tor1 {
         }
     }
 
-    impl<SC: StreamCipher, D: Digest + Clone> InboundClientLayer for CryptState<SC, D> {
+    impl<SC: StreamCipher, D: Digest + Clone, RCF: RelayCellFormatTrait> InboundClientLayer
+        for CryptState<SC, D, RCF>
+    {
         fn decrypt_inbound(&mut self, cell: &mut RelayCellBody) -> Option<&[u8]> {
             // This is a single iteration of the loop described in tor-spec
             // 5.5.3, "routing to the origin."
             self.cipher.apply_keystream(&mut cell.0[..]);
-            if cell.recognized(&mut self.digest, &mut self.last_digest_val) {
+            if cell.is_recognized::<_, RCF>(&mut self.digest, &mut self.last_digest_val) {
                 Some(&self.last_digest_val[..SENDME_TAG_LEN])
             } else {
                 None
@@ -441,22 +464,46 @@ pub(crate) mod tor1 {
     ///
     /// These operations is described in tor-spec section 6.1 "Relay cells"
     impl RelayCellBody {
+        /// Returns the byte slice of the `recognized` field.
+        fn recognized<RCF: RelayCellFormatTrait>(&self) -> &[u8] {
+            &self.0[RCF::FIELDS::RECOGNIZED_RANGE]
+        }
+        /// Returns the mut byte slice of the `recognized` field.
+        fn recognized_mut<RCF: RelayCellFormatTrait>(&mut self) -> &mut [u8] {
+            &mut self.0[RCF::FIELDS::RECOGNIZED_RANGE]
+        }
+        /// Returns the byte slice of the `digest` field.
+        fn digest<RCF: RelayCellFormatTrait>(&self) -> &[u8] {
+            &self.0[RCF::FIELDS::DIGEST_RANGE]
+        }
+        /// Returns the mut byte slice of the `digest` field.
+        fn digest_mut<RCF: RelayCellFormatTrait>(&mut self) -> &mut [u8] {
+            &mut self.0[RCF::FIELDS::DIGEST_RANGE]
+        }
         /// Prepare a cell body by setting its digest and recognized field.
-        fn set_digest<D: Digest + Clone>(
+        fn set_digest<D: Digest + Clone, RCF: RelayCellFormatTrait>(
             &mut self,
             d: &mut D,
             used_digest: &mut GenericArray<u8, D::OutputSize>,
         ) {
-            self.0[1..3].fill(0); // Set 'Recognized' to zero
-            self.0[5..9].fill(0); // Set Digest to zero
+            self.recognized_mut::<RCF>().fill(0); // Set 'Recognized' to zero
+            self.digest_mut::<RCF>().fill(0); // Set Digest to zero
 
             d.update(&self.0[..]);
             // TODO(nickm) can we avoid this clone?  Probably not.
             *used_digest = d.clone().finalize();
-            self.0[5..9].copy_from_slice(&used_digest[0..4]);
+            let used_digest_prefix = &used_digest[0..RCF::FIELDS::DIGEST_RANGE.len()];
+            self.digest_mut::<RCF>().copy_from_slice(used_digest_prefix);
         }
-        /// Check a cell to see whether its recognized field is set.
-        fn recognized<D: Digest + Clone>(
+        /// Check whether this just-decrypted cell is now an authenticated plaintext.
+        ///
+        /// This method returns true if the `recognized` field is all zeros, and if the
+        /// `digest` field is a digest of the correct material.
+        ///
+        /// If this method returns false, then either further decryption is required,
+        /// or the cell is corrupt.
+        // TODO #1336: Further optimize and/or benchmark this.
+        fn is_recognized<D: Digest + Clone, RCF: RelayCellFormatTrait>(
             &self,
             d: &mut D,
             rcvd: &mut GenericArray<u8, D::OutputSize>,
@@ -464,12 +511,7 @@ pub(crate) mod tor1 {
             use crate::util::ct;
 
             // Validate 'Recognized' field
-            let recognized = u16::from_be_bytes(
-                self.0[1..3]
-                    .try_into()
-                    .expect("Two-byte field was not two bytes!?"),
-            );
-            if recognized != 0 {
+            if !ct::is_zero(self.recognized::<RCF>()) {
                 return false;
             }
 
@@ -477,17 +519,20 @@ pub(crate) mod tor1 {
 
             let mut dtmp = d.clone();
             // Add bytes up to the 'Digest' field
-            dtmp.update(&self.0[..5]);
+            dtmp.update(&self.0[..RCF::FIELDS::DIGEST_RANGE.start]);
             // Add zeroes where the 'Digest' field is
-            dtmp.update([0_u8; 4]);
+            dtmp.update(RCF::FIELDS::EMPTY_DIGEST);
             // Add the rest of the bytes
-            dtmp.update(&self.0[9..]);
+            dtmp.update(&self.0[RCF::FIELDS::DIGEST_RANGE.end..]);
             // Clone the digest before finalize destroys it because we will use
             // it in the future
             let dtmp_clone = dtmp.clone();
             let result = dtmp.finalize();
 
-            if ct::bytes_eq(&self.0[5..9], &result[0..4]) {
+            if ct::bytes_eq(
+                self.digest::<RCF>(),
+                &result[0..RCF::FIELDS::DIGEST_RANGE.len()],
+            ) {
                 // Copy useful things out of this cell (we keep running digest)
                 *d = dtmp_clone;
                 *rcvd = result;
@@ -505,6 +550,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -517,11 +563,13 @@ mod test {
     use rand::RngCore;
     use tor_basic_utils::test_rng::testing_rng;
     use tor_bytes::SecretBuf;
+    use tor_cell::relaycell::RelayCellFormatV0;
 
     fn add_layers(
         cc_out: &mut OutboundClientCrypt,
         cc_in: &mut InboundClientCrypt,
-        pair: Tor1RelayCrypto,
+        // TODO #1067: test other formats
+        pair: Tor1RelayCrypto<RelayCellFormatV0>,
     ) {
         let (outbound, inbound, _) = pair.split();
         cc_out.add_layer(Box::new(outbound));
@@ -552,9 +600,9 @@ mod test {
         assert_eq!(cc_in.n_layers(), 3);
         assert_eq!(cc_out.n_layers(), 3);
 
-        let mut r1 = Tor1RelayCrypto::construct(KGen::new(seed1)).unwrap();
-        let mut r2 = Tor1RelayCrypto::construct(KGen::new(seed2)).unwrap();
-        let mut r3 = Tor1RelayCrypto::construct(KGen::new(seed3)).unwrap();
+        let mut r1 = Tor1RelayCrypto::<RelayCellFormatV0>::construct(KGen::new(seed1)).unwrap();
+        let mut r2 = Tor1RelayCrypto::<RelayCellFormatV0>::construct(KGen::new(seed2)).unwrap();
+        let mut r3 = Tor1RelayCrypto::<RelayCellFormatV0>::construct(KGen::new(seed3)).unwrap();
 
         let mut rng = testing_rng();
         for _ in 1..300 {
@@ -627,11 +675,11 @@ mod test {
 
         let mut cc_out = OutboundClientCrypt::new();
         let mut cc_in = InboundClientCrypt::new();
-        let pair = Tor1RelayCrypto::initialize(&K1[..]).unwrap();
+        let pair = Tor1RelayCrypto::<RelayCellFormatV0>::initialize(&K1[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
-        let pair = Tor1RelayCrypto::initialize(&K2[..]).unwrap();
+        let pair = Tor1RelayCrypto::<RelayCellFormatV0>::initialize(&K2[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
-        let pair = Tor1RelayCrypto::initialize(&K3[..]).unwrap();
+        let pair = Tor1RelayCrypto::<RelayCellFormatV0>::initialize(&K3[..]).unwrap();
         add_layers(&mut cc_out, &mut cc_in, pair);
 
         let mut xof = tor_llcrypto::d::Shake256::default();
